@@ -402,26 +402,6 @@ acm_open (const guint16 tag)
 }
 
 
-/*  ja:Codecを閉じる
-    acm_object,AcmObject
-           RET,TRUE:正常終了,FALSE:エラー                                   */
-gboolean
-acm_close (AcmObject *acm_object)
-{
-  if (acm_object && --(acm_object->counter) <= 0)
-    {
-#if defined (G_OS_WIN32) || defined (USE_W32LDR)
-      if (acm_object->stat != 0)
-        acm_convert_end (acm_object, NULL, NULL);
-      CloseDriver (acm_object->hDrvr, 0, 0);
-#endif /* defined (G_OS_WIN32) || defined (USE_W32LDR) */
-      g_free (acm_object->name);
-      g_free (acm_object);
-    }
-  return TRUE;
-}
-
-
 /*  ja:Codecの参照数を増やす
     acm_object,AcmObject
            RET,TRUE:正常終了,FALSE:エラー                                   */
@@ -438,11 +418,21 @@ acm_ref (AcmObject *acm_object)
 gboolean
 acm_unref (AcmObject *acm_object)
 {
-  return acm_close (acm_object);
+  if (acm_object && --(acm_object->counter) <= 0)
+    {
+#if defined (G_OS_WIN32) || defined (USE_W32LDR)
+      if (acm_object->stat != 0)
+        acm_convert_end (acm_object, NULL, NULL);
+      CloseDriver (acm_object->hDrvr, 0, 0);
+#endif /* defined (G_OS_WIN32) || defined (USE_W32LDR) */
+      g_free (acm_object->name);
+      g_free (acm_object);
+    }
+  return TRUE;
 }
 
 
-/*  ja:Codecをコピーする
+/*  ja:Codecを複製する
     acm_object,AcmObject
            RET,AcmObject,NULL:エラー                                        */
 AcmObject *
@@ -703,25 +693,42 @@ acm_convert_begin (AcmObject          *acm_object,
       return FALSE;
     }
   acm_object->acstmsize.cbStruct = sizeof (ACMDRVSTREAMSIZE);
-  acm_object->acstmsize.fdwSize = ACM_STREAMSIZEF_SOURCE;
-  acm_object->acstmsize.cbSrcLength
-                    = MAX (ACM_BUFFER_SIZE / wfx_get_block_align (wfx_in), 1)
-                                                * wfx_get_block_align (wfx_in);
-  acm_object->acstmsize.cbDstLength = 0;
+  acm_object->acstmsize.fdwSize = ACM_STREAMSIZEF_DESTINATION;
+  acm_object->acstmsize.cbSrcLength = 0;
+  acm_object->acstmsize.cbDstLength
+                    = MAX (ACM_BUFFER_SIZE / wfx_get_block_align (wfx_out), 1)
+                                            * wfx_get_block_align (wfx_out);
   if (SendDriverMessage (acm_object->hDrvr, ACMDM_STREAM_SIZE,
                 GPOINTER_TO_INT (&acm_object->acstminst),
                 GPOINTER_TO_INT (&acm_object->acstmsize)) != MMSYSERR_NOERROR
                                     || acm_object->acstmsize.cbSrcLength <= 0
                                     || acm_object->acstmsize.cbDstLength <= 0)
     {
-      SendDriverMessage (acm_object->hDrvr, ACMDM_STREAM_CLOSE,
+      acm_object->acstmsize.cbStruct = sizeof (ACMDRVSTREAMSIZE);
+      acm_object->acstmsize.fdwSize = ACM_STREAMSIZEF_SOURCE;
+      acm_object->acstmsize.cbSrcLength
+                    = MAX (ACM_BUFFER_SIZE / wfx_get_block_align (wfx_in), 1)
+                                                * wfx_get_block_align (wfx_in);
+      acm_object->acstmsize.cbDstLength = 0;
+      if (SendDriverMessage (acm_object->hDrvr, ACMDM_STREAM_SIZE,
+                GPOINTER_TO_INT (&acm_object->acstminst),
+                GPOINTER_TO_INT (&acm_object->acstmsize)) != MMSYSERR_NOERROR
+                                    || acm_object->acstmsize.cbSrcLength <= 0
+                                    || acm_object->acstmsize.cbDstLength <= 0)
+        {
+          SendDriverMessage (acm_object->hDrvr, ACMDM_STREAM_CLOSE,
                                 GPOINTER_TO_INT (&acm_object->acstminst), 0);
-      g_free (acm_object->acstminst.pwfxSrc);
-      g_free (acm_object->acstminst.pwfxDst);
-      acm_object->acstminst.pwfxSrc = NULL;
-      acm_object->acstminst.pwfxDst = NULL;
-      return FALSE;
+          g_free (acm_object->acstminst.pwfxSrc);
+          g_free (acm_object->acstminst.pwfxDst);
+          acm_object->acstminst.pwfxSrc = NULL;
+          acm_object->acstminst.pwfxDst = NULL;
+          return FALSE;
+        }
     }
+  acm_object->acstmsize.cbSrcLength = MAX (acm_object->acstmsize.cbSrcLength
+        / wfx_get_block_align (wfx_in), 1) * wfx_get_block_align (wfx_in);
+  acm_object->acstmsize.cbDstLength = MAX (acm_object->acstmsize.cbDstLength
+        / wfx_get_block_align (wfx_out), 1) * wfx_get_block_align (wfx_out);
   g_memset (&acm_object->acstmhdr, 0, sizeof (ACMDRVSTREAMHEADER));
   acm_object->acstmhdr.cbStruct = sizeof (ACMDRVSTREAMHEADER);
   acm_object->acstmhdr.pbSrc = g_malloc (acm_object->acstmsize.cbSrcLength);
@@ -866,8 +873,8 @@ acm_convert (AcmObject      *acm_object,
       acm_object->acstmhdr.cbSrcLength += leng;
       acm_object->acstmhdr.cbSrcLengthUsed = 0;
       acm_object->acstmhdr.cbDstLengthUsed = 0;
-      acm_object->acstmhdr.fdwConvert = acm_object->stat == 1
-                                                ? ACM_STREAMCONVERTF_START : 0;
+      acm_object->acstmhdr.fdwConvert = ACM_STREAMCONVERTF_BLOCKALIGN
+                    | (acm_object->stat == 1 ? ACM_STREAMCONVERTF_START : 0);
       acm_object->stat = 2;
       in_bytes -= leng;
       in_p += leng;
