@@ -1,6 +1,6 @@
 /*
     avicore
-    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2006 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -38,17 +38,23 @@ extern "C" {
 #define AVI_TYPE_UNKNOWN    0
 #define AVI_TYPE_VIDEO      1
 #define AVI_TYPE_AUDIO      2
-#define AVI_FILE_UNKNOWN    (-1)
-#define AVI_FILE_AVI        0
-#define AVI_FILE_BITMAP     1
-#define AVI_FILE_WAVE       2
-#define AVI_FILE_SCENARIO   3
+#define AVI_FILE_UNKNOWN    0
+#define AVI_FILE_AVI        1
+#define AVI_FILE_GSR        2
+#define AVI_FILE_BITMAP     3
+#define AVI_FILE_WAVE       4
+#define AVI_FILE_SCENARIO   5
 
 
 typedef struct _AviEdit AviEdit;
 typedef struct _AviFrame AviFrame;
 typedef struct _AviPcm AviPcm;
 typedef struct _AviSave AviSave;
+typedef struct _AviLanguage
+{
+  const gchar *name;
+  guint16 id;
+} AviLanguage;
 
 
 /******************************************************************************
@@ -75,209 +81,209 @@ avi_exit (void);
     avi_edit,AVI編集ハンドル
          RET,Rate                                                           */
 guint32
-avi_get_rate (AviEdit *avi_edit);
+avi_edit_get_rate (AviEdit *avi_edit);
 
 
 /*  ja:AVIのScaleを取得する
     avi_edit,AVI編集ハンドル
          RET,Scale                                                          */
 guint32
-avi_get_scale (AviEdit *avi_edit);
+avi_edit_get_scale (AviEdit *avi_edit);
 
 
 /*  ja:AVIの優先度を取得する
     avi_edit,AVI編集ハンドル
          RET,優先度                                                         */
 guint16
-avi_get_priority (AviEdit *avi_edit);
+avi_edit_get_priority (AviEdit *avi_edit);
 
 
 /*  ja:AVIの言語を取得する
     avi_edit,AVI編集ハンドル
          RET,言語                                                           */
 guint16
-avi_get_language (AviEdit *avi_edit);
+avi_edit_get_language (AviEdit *avi_edit);
 
 
 /*  ja:AVIの名前を取得する
     avi_edit,AVI編集ハンドル
          RET,名前                                                           */
 const gchar *
-avi_get_name (AviEdit *avi_edit);
+avi_edit_get_name (AviEdit *avi_edit);
 
 
 /*  ja:AVIの幅を取得する
     avi_edit,AVI編集ハンドル
          RET,幅,0:エラー                                                    */
 gint32
-avi_get_width (AviEdit *avi_edit);
+avi_edit_get_width (AviEdit *avi_edit);
 
 
 /*  ja:AVIの高さを取得する
     avi_edit,AVI編集ハンドル
          RET,高さ,0:エラー                                                  */
 gint32
-avi_get_height (AviEdit *avi_edit);
+avi_edit_get_height (AviEdit *avi_edit);
 
 
 /*  ja:AVIの色数を取得する
     avi_edit,AVI編集ハンドル
          RET,色数,0:エラー                                                  */
 guint16
-avi_get_bit_count (AviEdit *avi_edit);
+avi_edit_get_bit_count (AviEdit *avi_edit);
 
 
 /*  ja:AVIのX解像度を取得する
     avi_edit,AVI編集ハンドル
          RET,X解像度,-1:エラー                                              */
 gint32
-avi_get_x_pixels_per_meter (AviEdit *avi_edit);
+avi_edit_get_x_pixels_per_meter (AviEdit *avi_edit);
 
 
 /*  ja:AVIのY解像度を取得する
     avi_edit,AVI編集ハンドル
          RET,Y解像度,-1:エラー                                              */
 gint32
-avi_get_y_pixels_per_meter (AviEdit *avi_edit);
+avi_edit_get_y_pixels_per_meter (AviEdit *avi_edit);
 
 
 /*  ja:AVIのレートを取得する
     avi_edit,AVI編集ハンドル
          RET,レート,0:エラー                                                */
 guint32
-avi_get_samples_per_sec (AviEdit *avi_edit);
+avi_edit_get_samples_per_sec (AviEdit *avi_edit);
 
 
 /*  ja:AVIのチャンネルを取得する
     avi_edit,AVI編集ハンドル
          RET,チャンネル,0:エラー                                            */
 guint16
-avi_get_channels (AviEdit *avi_edit);
+avi_edit_get_channels (AviEdit *avi_edit);
 
 
 /*  ja:AVIのビットを取得する
     avi_edit,AVI編集ハンドル
          RET,ビット,0:エラー                                                */
 guint16
-avi_get_bits_per_sample (AviEdit *avi_edit);
+avi_edit_get_bits_per_sample (AviEdit *avi_edit);
 
 
 /*  ja:AVIのRateを設定する
     avi_edit,AVI編集ハンドル
         rate,Rate                                                           */
 void
-avi_set_rate (AviEdit *avi_edit,
-              guint32  rate);
+avi_edit_set_rate (AviEdit *avi_edit,
+                   guint32  rate);
 
 
 /*  ja:AVIのScaleを設定する
     avi_edit,AVI編集ハンドル
        scale,Scale                                                          */
 void
-avi_set_scale (AviEdit *avi_edit,
-               guint32  scale);
+avi_edit_set_scale (AviEdit *avi_edit,
+                    guint32  scale);
 
 
 /*  ja:AVIの優先度を設定する
     avi_edit,AVI編集ハンドル
     priority,優先度                                                         */
 void
-avi_set_priority (AviEdit *avi_edit,
-                  guint16  priority);
+avi_edit_set_priority (AviEdit *avi_edit,
+                       guint16  priority);
 
 
 /*  ja:AVIの言語を設定する
     avi_edit,AVI編集ハンドル
     language,言語                                                           */
 void
-avi_set_language (AviEdit *avi_edit,
-                  guint16  language);
+avi_edit_set_language (AviEdit *avi_edit,
+                       guint16  language);
 
 
 /*  ja:AVIの名前を設定する
     avi_edit,AVI編集ハンドル
         name,名前                                                           */
 void
-avi_set_name (AviEdit     *avi_edit,
-              const gchar *name);
+avi_edit_set_name (AviEdit     *avi_edit,
+                   const gchar *name);
 
 
 /*  ja:AVIの幅を設定する
     avi_edit,AVI編集ハンドル
        width,幅                                                             */
 void
-avi_set_width (AviEdit *avi_edit,
-               gint32   width);
+avi_edit_set_width (AviEdit *avi_edit,
+                    gint32   width);
 
 
 /*  ja:AVIの高さを設定する
     avi_edit,AVI編集ハンドル
       height,高さ                                                           */
 void
-avi_set_height (AviEdit *avi_edit,
-                gint32   height);
+avi_edit_set_height (AviEdit *avi_edit,
+                     gint32   height);
 
 
 /*  ja:AVIの色数を設定する
      avi_edit,AVI編集ハンドル
     bit_count,色数                                                          */
 void
-avi_set_bit_count (AviEdit *avi_edit,
-                   guint16  bit_count);
+avi_edit_set_bit_count (AviEdit *avi_edit,
+                        guint16  bit_count);
 
 
 /*  ja:AVIのX解像度を設定する
               avi_edit,AVI編集ハンドル
     x_pixels_per_meter,X解像度                                              */
 void
-avi_set_x_pixels_per_meter (AviEdit *avi_edit,
-                            gint32   x_pixels_per_meter);
+avi_edit_set_x_pixels_per_meter (AviEdit *avi_edit,
+                                 gint32   x_pixels_per_meter);
 
 
 /*  ja:AVIのY解像度を設定する
               avi_edit,AVI編集ハンドル
     y_pixels_per_meter,Y解像度                                              */
 void
-avi_set_y_pixels_per_meter (AviEdit *avi_edit,
-                            gint32   y_pixels_per_meter);
+avi_edit_set_y_pixels_per_meter (AviEdit *avi_edit,
+                                 gint32   y_pixels_per_meter);
 
 
 /*  ja:AVIのレートを設定する
            avi_edit,AVI編集ハンドル
     samples_per_sec,レート                                                  */
 void
-avi_set_samples_per_sec (AviEdit *avi_edit,
-                         guint32  samples_per_sec);
+avi_edit_set_samples_per_sec (AviEdit *avi_edit,
+                              guint32  samples_per_sec);
 
 
 /*  ja:AVIのチャンネルを設定する
     avi_edit,AVI編集ハンドル
     channels,チャンネル                                                     */
 void
-avi_set_channels (AviEdit *avi_edit,
-                  guint16  channels);
+avi_edit_set_channels (AviEdit *avi_edit,
+                       guint16  channels);
 
 
 /*  ja:AVIのビットを設定する
            avi_edit,AVI編集ハンドル
     bits_per_sample,ビット                                                  */
 void
-avi_set_bits_per_sample (AviEdit *avi_edit,
-                         guint16  bits_per_sample);
+avi_edit_set_bits_per_sample (AviEdit *avi_edit,
+                              guint16  bits_per_sample);
 
 
 /*  ja:AVIのタイプを求める
     avi_edit,AVI編集ハンドル
          RET,AVI_TYPE_VIDEO/AVI_TYPE_AUDIO/AVI_TYPE_UNKNOWN                 */
 gint
-avi_type (AviEdit *avi_edit);
+avi_edit_type (AviEdit *avi_edit);
 
 
 /*  ja:AVIの長さを求める
     avi_edit,AVI編集ハンドル
          RET,サンプル数                                                     */
 gint
-avi_length (AviEdit *avi_edit);
+avi_edit_length (AviEdit *avi_edit);
 
 
 /******************************************************************************
@@ -290,8 +296,8 @@ avi_length (AviEdit *avi_edit);
       sample,サンプル番号
          RET,時間(ミリ秒)                                                   */
 gint
-avi_sample_to_time (AviEdit    *avi_edit,
-                    const gint  sample);
+avi_time_from_sample (AviEdit    *avi_edit,
+                      const gint  sample);
 
 
 /*  ja:AVIの時間からサンプル番号を求める
@@ -307,7 +313,7 @@ avi_time_to_sample (AviEdit    *avi_edit,
     avi_edit,AVI編集ハンドル
          RET,時間(ミリ秒)                                                   */
 gint
-avi_length_time (AviEdit *avi_edit);
+avi_time_length (AviEdit *avi_edit);
 
 
 /*  ja:AVIのサンプル番号から対応するサンプル番号を求める
@@ -316,11 +322,29 @@ avi_length_time (AviEdit *avi_edit);
     avi_edit1,対応先のAVI編集ハンドル
           RET,対応先のサンプル番号                                          */
 gint
-avi_sample_to_sample (AviEdit    *avi_edit0,
+avi_time_sync_sample (AviEdit    *avi_edit0,
                       const gint  sample,
                       AviEdit    *avi_edit1);
 
 
+/******************************************************************************
+*                                                                             *
+* ja:AVI言語関数                                                              *
+*                                                                             *
+******************************************************************************/
+/*  ja:言語IDと名前を求める
+    RET,言語IDと名前                                                        */
+const AviLanguage *
+avi_lang_get_struct (void);
+
+
+/*  ja:言語IDから名前を求める
+     ID,言語ID
+    RET,名前                                                                */
+const gchar *
+avi_lang_get_name (const guint16 id);
+
+
 /******************************************************************************
 *                                                                             *
 * ja:AVI入力関数                                                              *
@@ -330,30 +354,35 @@ avi_sample_to_sample (AviEdit    *avi_edit0,
     file,ファイル名
      RET,AVI_TYPE_*                                                         */
 gint
-avi_file_type (const gchar *file);
+avi_main_type (const gchar *file);
 
 
-/*  ja:AVIのストリームの数を取得する
+/*  ja:ストリームの数を取得する
     file,ファイル名
      RET,ストリームの数,0:エラー                                            */
-gint32
+gint
 avi_main_streams (const gchar *file);
 
 
 /*  ja:AVIをファイルから開く
-     file,ファイル名
-    param,ストリーム番号
-      RET,AVI編集ハンドル,NULL:エラー                                       */
-AviEdit *
-avi_open (const gchar *file,
-          const gint   param);
+    file,ファイル名
+     RET,AVI編集ハンドル,NULL:エラー                                        */
+AviEdit **
+avi_edit_open (const gchar *file);
 
 
 /*  ja:AVIを閉じる
     avi_edit,AVI編集ハンドル
          RET,TRUE:正常終了,FALSE:エラー                                     */
 gboolean
-avi_release (AviEdit *avi_edit);
+avi_edit_close (AviEdit *avi_edit);
+
+
+/*  ja:AVIを複製する
+    avi_edit,AVI編集ハンドル
+         RET,AVI編集ハンドル                                                */
+#define avi_edit_dup(avi_edit)                                              \
+                            avi_edit_copy(avi_edit,0,avi_edit_length(avi_edit))
 
 
 /******************************************************************************
@@ -365,14 +394,14 @@ avi_release (AviEdit *avi_edit);
     avi_edit,AVI編集ハンドル
          RET,AVIフレーム構造体,NULL:エラー                                  */
 AviFrame *
-avi_get_frame_open (AviEdit *avi_edit);
+avi_frame_open (AviEdit *avi_edit);
 
 
 /*  ja:DIBに展開されたフレームを閉じる
     avi_frame,AVIフレーム構造体
           RET,TRUE:正常終了,FALSE:エラー                                    */
 gboolean
-avi_get_frame_close (AviFrame *avi_frame);
+avi_frame_close (AviFrame *avi_frame);
 
 
 /*  ja:DIBに展開されたフレームを取得する
@@ -383,11 +412,11 @@ avi_get_frame_close (AviFrame *avi_frame);
     bit_count,色数
           RET,パックDIBへのポインタ,NULL:エラー                             */
 BitmapInfoHeader *
-avi_get_frame (AviFrame   *avi_frame,
-               const gint  pos,
-               const gint  width,
-               const gint  height,
-               const gint  bit_count);
+avi_frame_get_bitmap (AviFrame   *avi_frame,
+                      const gint  pos,
+                      const gint  width,
+                      const gint  height,
+                      const gint  bit_count);
 
 
 /*  ja:RAWに展開されたフレームを取得する
@@ -397,10 +426,10 @@ avi_get_frame (AviFrame   *avi_frame,
        height,高さ
           RET,RAWへのポインタ,NULL:エラー                               */
 guint8 *
-avi_get_frame32 (AviFrame   *avi_frame,
-                 const gint  pos,
-                 const gint  width, 
-                 const gint  height);
+avi_frame_get_raw (AviFrame   *avi_frame,
+                   const gint  pos,
+                   const gint  width,
+                   const gint  height);
 
 
 /*  ja:フレームがキーフレームか判定する
@@ -408,8 +437,8 @@ avi_get_frame32 (AviFrame   *avi_frame,
          pos,取得するフレーム(サンプル番号)
          RET,TRUE:キーフレーム,FALSE:非キーフレーム                         */
 gboolean
-avi_is_keyframe (AviEdit    *avi_edit,
-                 const gint  pos);
+avi_frame_is_key (AviEdit    *avi_edit,
+                  const gint  pos);
 
 
 /******************************************************************************
@@ -421,14 +450,14 @@ avi_is_keyframe (AviEdit    *avi_edit,
     avi_edit,AVI編集ハンドル
          RET,AVIPCM構造体,NULL:エラー                                       */
 AviPcm *
-avi_get_pcm_open (AviEdit *avi_edit);
+avi_pcm_open (AviEdit *avi_edit);
 
 
 /*  ja:PCMに展開されたオーディオを閉じる
     avi_pcm,AVIPCM構造体
         RET,TRUE:正常終了,FALSE:エラー                                      */
 gboolean
-avi_get_pcm_close (AviPcm *avi_pcm);
+avi_pcm_close (AviPcm *avi_pcm);
 
 
 /*  ja:PCMに展開されたオーディオを取得する
@@ -439,11 +468,11 @@ avi_get_pcm_close (AviPcm *avi_pcm);
     out_samples,PCMデータの実サンプル数
             RET,TRUE:正常終了,FALSE:エラー                                  */
 gboolean
-avi_get_pcm (AviPcm     *avi_pcm,
-             const gint  start,
-             const gint  samples,
-             gpointer   *out_buf,
-             gint       *out_samples);
+avi_pcm_get_raw (AviPcm      *avi_pcm,
+                 const gint   start,
+                 const gsize  samples,
+                 gpointer    *out_buf,
+                 gsize       *out_samples);
 
 
 /******************************************************************************
@@ -457,9 +486,9 @@ avi_get_pcm (AviPcm     *avi_pcm,
      samples,サンプル数
          RET,TRUE:正常終了,FALSE:エラー                                     */
 gboolean
-avi_delete (AviEdit    *avi_edit,
-            const gint  start,
-            const gint  samples);
+avi_edit_delete (AviEdit    *avi_edit,
+                 const gint  start,
+                 const gint  samples);
 
 
 /*  ja:AVIをコピーする
@@ -468,9 +497,9 @@ avi_delete (AviEdit    *avi_edit,
      samples,サンプル数
          RET,AVI編集ハンドル,NULL:エラー                                    */
 AviEdit *
-avi_copy (AviEdit    *avi_edit,
-          const gint  start,
-          const gint  samples);
+avi_edit_copy (AviEdit    *avi_edit,
+               const gint  start,
+               const gint  samples);
 
 
 /*  ja:AVIを貼り付ける
@@ -479,15 +508,9 @@ avi_copy (AviEdit    *avi_edit,
     avi_src,貼り付けるAVI編集ハンドル
         RET,TRUE:正常終了,FALSE:エラー                                      */
 gboolean
-avi_paste (AviEdit    *avi_dst,
-           const gint  pos,
-           AviEdit    *avi_src);
-
-
-/*  ja:AVIを複製する
-    avi_edit,AVI編集ハンドル
-         RET,AVI編集ハンドル                                                */
-#define avi_clone(avi_edit) avi_copy(avi_edit,0,avi_length(avi_edit))
+avi_edit_paste (AviEdit    *avi_dst,
+                const gint  pos,
+                AviEdit    *avi_src);
 
 
 /******************************************************************************
@@ -500,15 +523,15 @@ avi_paste (AviEdit    *avi_dst,
     interleave,TRUE:インターリーブする,FALSE:インターリーブしない
            RET,シナリオオブジェクト,NULL:エラー                             */
 gchar *
-avi_to_scenario (AviEdit        *avi_edit[],
-                 const gboolean  interleave);
+avi_edit_to_scenario (AviEdit        **avi_edit,
+                      const gboolean   interleave);
 
 
 /*  ja:シナリオオブジェクトからAVIを作る
     scenario,シナリオオブジェクト
          RET,AVI編集ハンドルへのポインタ,NULL:エラー                        */
 AviEdit **
-avi_from_scenario (const gchar *scenario);
+avi_edit_from_scenario (const gchar *scenario);
 
 
 /******************************************************************************
@@ -523,6 +546,13 @@ gboolean
 avi_save_free (AviSave *avi_save);
 
 
+/*  ja:保存されるファイルのタイプを返す
+    avi_save,オプション
+         RET,ファイルタイプ,-1:エラー                                       */
+gint
+avi_save_type (AviSave *avi_save);
+
+
 /*  ja:すべて保存されるかチェックする
     avi_edit,AVI編集ハンドル
     avi_save,オプション
@@ -574,7 +604,7 @@ avi_save_wave (AviEdit             *avi_edit,
     user_data,データ
           RET,TRUE:正常終了,FALSE:エラー                                    */
 gboolean
-avi_save_with_options (AviEdit      *avi_edit[],
+avi_save_with_options (AviEdit     **avi_edit,
                        const gchar  *file,
                        AviSave      *avi_save,
                        gboolean    (*func)(gint, gpointer),
