@@ -1,6 +1,6 @@
 /*
     avicore
-    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2006 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,6 +19,36 @@
 #include "wave.h"
 
 
+/******************************************************************************
+*                                                                             *
+* ja:PCM生成関数                                                              *
+*                                                                             *
+******************************************************************************/
+/*  ja:無音PCMを作る
+        wfx,WaveFormatEx構造体へのポインタ
+    samples,出力データのサンプル数
+        RET,無音のPCM                                                       */
+gpointer
+wave_fill_silent_pcm (WaveFormatEx *wfx,
+                      const gsize   samples)
+{
+  gpointer buffer;
+  if (wfx_get_bits_per_sample (wfx) == 8)
+    {
+      gsize bytes;
+
+      bytes = samples * wfx_get_channels (wfx) * sizeof (guint8);
+      buffer = g_malloc (bytes);
+      g_memset (buffer, 128, bytes);
+    }
+  else
+    {
+      buffer = g_malloc0 (samples * wfx_get_channels (wfx) * sizeof (gint16));
+    }
+  return buffer;
+}
+
+
 /******************************************************************************
 *                                                                             *
 * ja:PCM加工関数                                                              *
@@ -57,10 +87,10 @@ wave_convert_channels_begin (WaveFormatEx *wfx0,
             RET,TRUE:正常終了,FALSE:エラー                                  */
 gboolean
 wave_convert_channels (WaveConvChannels *wcc,
-                       gpointer          in_buf,
-                       const gint        in_samples,
+                       gconstpointer     in_buf,
+                       const gsize       in_samples,
                        gpointer         *out_buf,
-                       gint             *out_samples)
+                       gsize            *out_samples)
 {
   gpointer result;
   gint i;
@@ -118,7 +148,7 @@ wave_convert_channels (WaveConvChannels *wcc,
 gboolean
 wave_convert_channels_end (WaveConvChannels *wcc,
                            gpointer         *out_buf,
-                           gint             *out_samples)
+                           gsize            *out_samples)
 {
   if (out_buf)
     *out_buf = NULL;
@@ -165,6 +195,7 @@ wave_convert_samples_per_sec_begin (WaveFormatEx *wfx0,
 }
 
 
+#include <windows.h>
 /*  ja:PCMのサンプリング周波数変換をする
             wcs,変換ハンドル
          in_buf,入力データ
@@ -174,21 +205,23 @@ wave_convert_samples_per_sec_begin (WaveFormatEx *wfx0,
             RET,TRUE:正常終了,FALSE:エラー                                  */
 gboolean
 wave_convert_samples_per_sec (WaveConvSamplesPerSec *wcs,
-                              gpointer               in_buf,
-                              const gint             in_samples,
+                              gconstpointer          in_buf,
+                              const gsize            in_samples,
                               gpointer              *out_buf,
-                              gint                  *out_samples)
+                              gsize                 *out_samples)
 {
   gpointer result;
-  gint i, j, d, block_align, samples, pos;
+  gsize samples;
+  gint i, j, d, block_align, pos;
 
   block_align = wcs->channels * wcs->bits_per_sample / 8;
-  samples = (glonglong)(wcs->in_samples + in_samples) * wcs->samples_per_sec1
-                                    / wcs->samples_per_sec0 - wcs->out_samples;
+  samples = (glonglong)((wcs->in_samples + in_samples) * wcs->samples_per_sec1
+    + wcs->samples_per_sec0 - 1) / wcs->samples_per_sec0 - wcs->out_samples;
   result = g_malloc (samples * block_align);
   if (wcs->bits_per_sample == 8)
     {
-      guint8 *i_buf, *o_buf;
+      const guint8 *i_buf;
+      guint8 *o_buf;
 
       i_buf = in_buf;
       o_buf = result;
@@ -215,7 +248,8 @@ wave_convert_samples_per_sec (WaveConvSamplesPerSec *wcs,
     }
   else
     {
-      gint16 *i_buf, *o_buf;
+      const gint16 *i_buf;
+      gint16 *o_buf;
 
       i_buf = in_buf;
       o_buf = result;
@@ -270,13 +304,14 @@ wave_convert_samples_per_sec (WaveConvSamplesPerSec *wcs,
 gboolean
 wave_convert_samples_per_sec_end (WaveConvSamplesPerSec *wcs,
                                   gpointer              *out_buf,
-                                  gint                  *out_samples)
+                                  gsize                 *out_samples)
 {
   gpointer result;
-  gint i, j, d, samples;
+  gsize samples;
+  gint i, j, d;
 
-  samples = (glonglong)wcs->in_samples * wcs->samples_per_sec1
-                                    / wcs->samples_per_sec0 - wcs->out_samples;
+  samples = (glonglong)(wcs->in_samples * wcs->samples_per_sec1
+    + wcs->samples_per_sec0 - 1) / wcs->samples_per_sec0 - wcs->out_samples;
   result = g_malloc (samples * (wcs->channels * wcs->bits_per_sample / 8));
   if (wcs->bits_per_sample == 8)
     {
@@ -359,10 +394,10 @@ wave_convert_bits_per_sample_begin (WaveFormatEx *wfx0,
             RET,TRUE:正常終了,FALSE:エラー                                  */
 gboolean
 wave_convert_bits_per_sample (WaveConvBitPerSample *wcb,
-                              gpointer              in_buf,
-                              const gint            in_samples,
+                              gconstpointer         in_buf,
+                              const gsize           in_samples,
                               gpointer             *out_buf,
-                              gint                 *out_samples)
+                              gsize                *out_samples)
 {
   gpointer result;
   gint i;
@@ -404,7 +439,7 @@ wave_convert_bits_per_sample (WaveConvBitPerSample *wcb,
 gboolean
 wave_convert_bits_per_sample_end (WaveConvBitPerSample *wcb,
                                   gpointer             *out_buf,
-                                  gint                 *out_samples)
+                                  gsize                *out_samples)
 {
   if (out_buf)
     *out_buf = NULL;
@@ -422,13 +457,13 @@ typedef struct _WaveConvGeneric *WaveConvGeneric;
 typedef WaveConvGeneric (*WaveConvBeginFunc)(WaveFormatEx *wfx0,
                                              WaveFormatEx *wfx1);
 typedef gboolean (*WaveConvFunc)(WaveConvGeneric  wch,
-                                 gpointer         in_buf,
-                                 gint             in_samples,
+                                 gconstpointer    in_buf,
+                                 const gsize      in_samples,
                                  gpointer        *out_buf,
-                                 gint            *out_samples);
+                                 gsize           *out_samples);
 typedef gboolean (*WaveConvEndFunc)(WaveConvGeneric  wch,
                                     gpointer        *out_buf,
-                                    gint            *out_samples);
+                                    gsize           *out_samples);
 typedef struct _WaveConvProccess
 {
   WaveConvBeginFunc func_begin;
@@ -565,13 +600,14 @@ wave_convert_begin (WaveFormatEx *wfx0,
             RET,TRUE:正常終了,FALSE:エラー                                  */
 gboolean
 wave_convert (WaveConv      *wcv,
-              gpointer       in_buf,
-              const gint     in_samples,
+              gconstpointer  in_buf,
+              const gsize    in_samples,
               gpointer      *out_buf,
-              gint          *out_samples)
+              gsize         *out_samples)
 {
   gpointer buf;
-  gint i, samples;
+  gsize samples;
+  gint i;
 
   buf = g_memdup (in_buf, in_samples * wcv->block_align0);
   samples = in_samples;
@@ -606,16 +642,17 @@ wave_convert (WaveConv      *wcv,
 gboolean
 wave_convert_end (WaveConv *wcv,
                   gpointer *out_buf,
-                  gint     *out_samples)
+                  gsize    *out_samples)
 {
   gboolean result = TRUE;
   gpointer buf = NULL;
-  gint i, samples = 0;
+  gsize samples = 0;
+  gint i;
 
   for (i = 0; i < wcv->procs; i++)
     {
       gpointer e_buf;
-      gint e_samples;
+      gsize e_samples;
       gint j;
 
       if (!wcv->proc[i].func_end (wcv->proc[i].handle, &e_buf, &e_samples))
@@ -660,13 +697,13 @@ wave_convert_end (WaveConv *wcv,
 gboolean
 wave_convert_direct (WaveFormatEx  *wfx0,
                      WaveFormatEx  *wfx1,
-                     gpointer       in_buf,
-                     const gint     in_samples,
+                     gconstpointer  in_buf,
+                     const gsize    in_samples,
                      gpointer      *out_buf,
-                     gint          *out_samples)
+                     gsize         *out_samples)
 {
   gpointer buf, e_buf;
-  gint samples, e_samples;
+  gsize samples, e_samples;
   WaveConv *wcv;
 
   wcv = wave_convert_begin (wfx0, wfx1);
