@@ -1,6 +1,6 @@
 /*
     Video maid
-    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2006 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,28 +16,24 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
+#include "change.h"
 #include "command.h"
 #include "file.h"
 #include "general.h"
 #include "jump.h"
+#include "merge.h"
+#include "preview.h"
 #include "prop.h"
 #include "sigfile.h"
 #include "size.h"
 #include "thread.h"
 #include "version.h"
-#if GTK_CHECK_VERSION(2,4,0)
-# include "avicore/avifileextra.h"
-#else /* not GTK_CHECK_VERSION(2,4,0) */
-# include "avicore/avifilesel.h"
-#endif /* not GTK_CHECK_VERSION(2,4,0) */
+#include "avicore/aviclip.h"
+#include "avicore/aviextra.h"
 #include "misc/fileio.h"
-#include "misc/misc.h"
 #include "orz/orzconf.h"
 #include "orz/orzhistory.h"
 #include "orz/orzmdi.h"
-#ifdef G_OS_WIN32
-# include <gdk/gdkwin32.h>
-#endif /* G_OS_WIN32 */
 
 
 /******************************************************************************
@@ -62,8 +58,7 @@ struct _CommandOpenOk
 {
   gboolean merge;
   gchar *file;
-  AviSave *avi_save;
-  GtkWidget *dialog;
+  GtkWidget *dialog, *check;
 };
 
 
@@ -77,11 +72,8 @@ command_open_ok (GtkWidget     *widget,
   file = gtk_file_selection_get_filename (GTK_FILE_SELECTION (cmd_ok->dialog));
   cmd_ok->file = file && file[0] != '\0'
                 && file[g_strlen (file) - 1] != '/' ? g_strdup (file) : NULL;
-  cmd_ok->merge = avi_file_selection_get_continual
-                                        (AVI_FILE_SELECTION (cmd_ok->dialog));
-  cmd_ok->avi_save = avi_file_selection_get_option
-                                        (AVI_FILE_SELECTION (cmd_ok->dialog));
-
+  cmd_ok->merge = gtk_toggle_button_get_active
+                                        (GTK_TOGGLE_BUTTON (cmd_ok->check));
   gtk_widget_destroy (cmd_ok->dialog);
 }
 #endif /* not GTK_CHECK_VERSION(2,4,0) */
@@ -91,35 +83,43 @@ void
 command_open (GtkWidget *widget,
               gpointer   user_data)
 {
-  gchar *path;
+  gchar *path, *title;
+  GtkStockItem stock_item;
   CommandOpenOk cmd_ok;
   FileOpen *file_open;
-#if GTK_CHECK_VERSION(2,4,0)
-  GtkWidget *check;
-#endif /* GTK_CHECK_VERSION(2,4,0) */
 
+  gtk_stock_lookup (GTK_STOCK_OPEN, &stock_item);
+  title = misc_mnemonic_to_text (stock_item.label);
   cmd_ok.file = NULL;
+  cmd_ok.merge = file_merge;
+  cmd_ok.check = gtk_check_button_new_with_label (_("Continual"));
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cmd_ok.check), file_merge);
 #if GTK_CHECK_VERSION(2,4,0)
-  cmd_ok.dialog = gtk_file_chooser_dialog_new (_("Open"),
+  cmd_ok.dialog = gtk_file_chooser_dialog_new (title,
                             GTK_WINDOW (window), GTK_FILE_CHOOSER_ACTION_OPEN,
+# ifdef G_OS_WIN32
                             GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
                             GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+# else /* not G_OS_WIN32 */
+                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                            GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+# endif /* not G_OS_WIN32 */
                             NULL);
   if (open_path)
     gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (cmd_ok.dialog),
                                                                     open_path);
-  check = gtk_check_button_new_with_label (_("Continual"));
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), file_merge);
-  gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (cmd_ok.dialog), check);
+  gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (cmd_ok.dialog),
+                                                                cmd_ok.check);
   if (gtk_dialog_run (GTK_DIALOG (cmd_ok.dialog)) == GTK_RESPONSE_ACCEPT)
     {
       cmd_ok.file = gtk_file_chooser_get_filename
                                             (GTK_FILE_CHOOSER (cmd_ok.dialog));
-      cmd_ok.merge = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check));
+      cmd_ok.merge = gtk_toggle_button_get_active
+                                            (GTK_TOGGLE_BUTTON (cmd_ok.check));
     }
   gtk_widget_destroy (cmd_ok.dialog);
 #else /* not GTK_CHECK_VERSION(2,4,0) */
-  cmd_ok.dialog = avi_file_selection_new (_("Open"));
+  cmd_ok.dialog = gtk_file_selection_new (title);
   g_signal_connect (G_OBJECT (cmd_ok.dialog), "destroy", gtk_main_quit, NULL);
   g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (cmd_ok.dialog)->ok_button),
                             "clicked", G_CALLBACK (command_open_ok), &cmd_ok);
@@ -135,13 +135,14 @@ command_open (GtkWidget *widget,
       g_free (path);
     }
   gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (cmd_ok.dialog));
-  avi_file_selection_set_continual (AVI_FILE_SELECTION (cmd_ok.dialog),
-                                    file_merge);
-  avi_file_selection_show_continual (AVI_FILE_SELECTION (cmd_ok.dialog));
+  gtk_box_pack_end (GTK_BOX (GTK_FILE_SELECTION (cmd_ok.dialog)->main_vbox),
+                                                cmd_ok.check, FALSE, FALSE, 0);
+  gtk_widget_show (cmd_ok.check);
   gtk_widget_show (cmd_ok.dialog);
   gtk_grab_add (cmd_ok.dialog);
   gtk_main ();
 #endif /* not GTK_CHECK_VERSION(2,4,0) */
+  g_free (title);
   if (cmd_ok.file)
     {
       g_free (open_path);
@@ -172,19 +173,49 @@ command_close (GtkWidget *widget,
 }
 
 
+typedef struct _CommandSaveOk CommandSaveOk;
+struct _CommandSaveOk
+{
+  gboolean addext;
+  gchar *file;
+  AviSave *avi_save;
+  GtkWidget *dialog, *check, *extra;
+};
+
+
+#if ! GTK_CHECK_VERSION(2,4,0)
+static void
+command_save_ok (GtkWidget     *widget,
+                 CommandSaveOk *cmd_ok)
+{
+  const gchar *file;
+
+  file = gtk_file_selection_get_filename (GTK_FILE_SELECTION (cmd_ok->dialog));
+  cmd_ok->file = file && file[0] != '\0'
+                && file[g_strlen (file) - 1] != '/' ? g_strdup (file) : NULL;
+  cmd_ok->avi_save = avi_extra_get_option (AVI_EXTRA (cmd_ok->extra));
+  cmd_ok->addext = gtk_toggle_button_get_active
+                                        (GTK_TOGGLE_BUTTON (cmd_ok->check));
+  gtk_widget_destroy (cmd_ok->dialog);
+}
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
+
+
 void
 command_save (GtkWidget *widget,
               gpointer   user_data)
 {
-  gchar *path;
+  gchar *path, *title;
   gint i = 0;
+  GtkStockItem stock_item;
   AviEdit *avi_edit[3];
-  CommandOpenOk cmd_ok;
+  CommandSaveOk cmd_ok;
   FileSave *file_save;
   VmaidWindow *vmaid;
-#if GTK_CHECK_VERSION(2,4,0)
-  GtkWidget *extra;
-#endif /* GTK_CHECK_VERSION(2,4,0) */
+  GtkWidget *vbox;
+#if ! GTK_CHECK_VERSION(2,4,0)
+  GtkWidget *hbox_base, *hbox_button, *vbox_base;
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
 
   vmaid = orz_mdi_get_data (ORZ_MDI (mdi), -1);
   if (vmaid->avi_edit[0])
@@ -192,32 +223,50 @@ command_save (GtkWidget *widget,
   if (vmaid->avi_edit[1])
     avi_edit[i++] = vmaid->avi_edit[1];
   avi_edit[i] = NULL;
+  gtk_stock_lookup (GTK_STOCK_SAVE_AS, &stock_item);
+  title = misc_mnemonic_to_text (stock_item.label);
+  cmd_ok.addext = addext;
   cmd_ok.file = NULL;
   cmd_ok.avi_save = NULL;
+  cmd_ok.extra = avi_extra_new ();
+  avi_extra_set_handle (AVI_EXTRA (cmd_ok.extra), avi_edit);
+  cmd_ok.check = gtk_check_button_new_with_mnemonic
+                                        (_("_Automatic file name extension"));
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cmd_ok.check), addext);
+  vbox = gtk_vbox_new (FALSE, SPACING);
+  gtk_container_set_border_width (GTK_CONTAINER (vbox), SPACING);
+  gtk_box_pack_start (GTK_BOX (vbox), cmd_ok.check, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), cmd_ok.extra, TRUE, TRUE, 0);
+  gtk_widget_show_all (vbox);
 #if GTK_CHECK_VERSION(2,4,0)
-  cmd_ok.dialog = gtk_file_chooser_dialog_new (_("Save As"),
+  cmd_ok.dialog = gtk_file_chooser_dialog_new (title,
                             GTK_WINDOW (window), GTK_FILE_CHOOSER_ACTION_SAVE,
+# ifdef G_OS_WIN32
                             GTK_STOCK_SAVE_AS, GTK_RESPONSE_ACCEPT,
                             GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+# else /* not G_OS_WIN32 */
+                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                            GTK_STOCK_SAVE_AS, GTK_RESPONSE_ACCEPT,
+# endif /* not G_OS_WIN32 */
                             NULL);
   if (save_path)
     gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (cmd_ok.dialog),
                                                                     save_path);
-  extra = avi_file_extra_new ();
-  avi_file_extra_set_handle (AVI_FILE_EXTRA (extra), avi_edit);
-  gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (cmd_ok.dialog), extra);
+  gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (cmd_ok.dialog), vbox);
   if (gtk_dialog_run (GTK_DIALOG (cmd_ok.dialog)) == GTK_RESPONSE_ACCEPT)
     {
       cmd_ok.file = gtk_file_chooser_get_filename
                                             (GTK_FILE_CHOOSER (cmd_ok.dialog));
-      cmd_ok.avi_save = avi_file_extra_get_option (AVI_FILE_EXTRA (extra));
+      cmd_ok.avi_save = avi_extra_get_option (AVI_EXTRA (cmd_ok.extra));
+      cmd_ok.addext = gtk_toggle_button_get_active
+                                            (GTK_TOGGLE_BUTTON (cmd_ok.check));
     }
   gtk_widget_destroy (cmd_ok.dialog);
 #else /* not GTK_CHECK_VERSION(2,4,0) */
-  cmd_ok.dialog = avi_file_selection_new (_("Save As"));
+  cmd_ok.dialog = gtk_file_selection_new (title);
   g_signal_connect (G_OBJECT (cmd_ok.dialog), "destroy", gtk_main_quit, NULL);
   g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (cmd_ok.dialog)->ok_button),
-                            "clicked", G_CALLBACK (command_open_ok), &cmd_ok);
+                            "clicked", G_CALLBACK (command_save_ok), &cmd_ok);
   g_signal_connect_swapped
                 (G_OBJECT (GTK_FILE_SELECTION (cmd_ok.dialog)->cancel_button),
                                     "clicked", G_CALLBACK (gtk_widget_destroy),
@@ -230,13 +279,64 @@ command_save (GtkWidget *widget,
       g_free (path);
     }
   gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (cmd_ok.dialog));
-  avi_file_selection_set_handle (AVI_FILE_SELECTION (cmd_ok.dialog), avi_edit);
+  /* ja:フレームとボックス */
+  hbox_button = gtk_widget_get_parent
+                            (GTK_FILE_SELECTION (cmd_ok.dialog)->ok_button);
+  gtk_widget_ref (hbox_button);
+  gtk_container_remove (GTK_CONTAINER (GTK_FILE_SELECTION
+                                    (cmd_ok.dialog)->main_vbox), hbox_button);
+  gtk_widget_ref (GTK_FILE_SELECTION (cmd_ok.dialog)->main_vbox);
+  gtk_container_remove (GTK_CONTAINER (cmd_ok.dialog),
+                                GTK_FILE_SELECTION (cmd_ok.dialog)->main_vbox);
+
+  hbox_base = gtk_hbox_new (FALSE, SPACING);
+  gtk_box_pack_start (GTK_BOX (hbox_base),
+                GTK_FILE_SELECTION (cmd_ok.dialog)->main_vbox, TRUE, TRUE, 0);
+  gtk_box_pack_end (GTK_BOX (hbox_base), vbox, TRUE, TRUE, 0);
+  gtk_widget_show (hbox_base);
+
+  vbox_base = gtk_vbox_new (FALSE, SPACING);
+  gtk_box_pack_start (GTK_BOX (vbox_base), hbox_base, TRUE, TRUE, 0);
+  gtk_box_pack_end (GTK_BOX (vbox_base), hbox_button, FALSE, FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (cmd_ok.dialog), vbox_base);
+  gtk_widget_show (vbox_base);
+
   gtk_widget_show (cmd_ok.dialog);
   gtk_grab_add (cmd_ok.dialog);
   gtk_main ();
 #endif /* not GTK_CHECK_VERSION(2,4,0) */
+  g_free (title);
   if (cmd_ok.file && cmd_ok.avi_save)
     {
+      if (cmd_ok.addext)
+        {
+          gchar *basename;
+
+          basename = g_path_get_basename (cmd_ok.file);
+          if (!g_strchr (basename, '.'))
+            {
+              const gchar *ext;
+
+              switch (avi_save_type (cmd_ok.avi_save))
+                {
+                  case AVI_FILE_AVI:      ext = ".avi"; break;
+                  case AVI_FILE_BITMAP:   ext = ".bmp"; break;
+                  case AVI_FILE_WAVE:     ext = ".wav"; break;
+                  case AVI_FILE_SCENARIO: ext = ".scn"; break;
+                  default:                ext = NULL;
+                }
+              if (ext)
+                {
+                  gchar *tmp;
+
+                  tmp = g_strconcat (cmd_ok.file, ext, NULL);
+                  g_free (cmd_ok.file);
+                  cmd_ok.file = tmp;
+                }
+            }
+          g_free (basename);
+        }
+      addext = cmd_ok.addext;
       g_free (save_path);
       path = fileio_get_full_path (cmd_ok.file);
       save_path = g_path_get_dirname (path);
@@ -254,7 +354,7 @@ command_save (GtkWidget *widget,
       file_save = g_malloc (sizeof (FileSave));
       file_save->file = cmd_ok.file;
       for (i = 0; avi_edit[i]; i++)
-        file_save->avi_edit[i] = avi_clone (avi_edit[i]);
+        file_save->avi_edit[i] = avi_edit_dup (avi_edit[i]);
       file_save->avi_edit[i] = NULL;
       file_save->avi_save = cmd_ok.avi_save;
       /* ja:保存 */
@@ -272,6 +372,14 @@ command_save (GtkWidget *widget,
 }
 
 
+void
+command_preview (GtkWidget *widget,
+                 gpointer   user_data)
+{
+  preview ((VmaidWindow *)orz_mdi_get_data (ORZ_MDI (mdi), -1));
+}
+
+
 void
 command_property (GtkWidget *widget,
                   gpointer   user_data)
@@ -285,17 +393,17 @@ command_property (GtkWidget *widget,
   propcfg.scale = vmaid->scale;
   if (vmaid->avi_edit[0])
     {
-      propcfg.priority0 = avi_get_priority (vmaid->avi_edit[0]);
-      propcfg.language0 = avi_get_language (vmaid->avi_edit[0]);
-      if (avi_get_name (vmaid->avi_edit[0]))
-        propcfg.name0 = g_strdup (avi_get_name (vmaid->avi_edit[0]));
-      propcfg.width = avi_get_width (vmaid->avi_edit[0]);
-      propcfg.height = avi_get_height (vmaid->avi_edit[0]);
-      propcfg.bit_count = avi_get_bit_count (vmaid->avi_edit[0]);
+      propcfg.priority0 = avi_edit_get_priority (vmaid->avi_edit[0]);
+      propcfg.language0 = avi_edit_get_language (vmaid->avi_edit[0]);
+      if (avi_edit_get_name (vmaid->avi_edit[0]))
+        propcfg.name0 = g_strdup (avi_edit_get_name (vmaid->avi_edit[0]));
+      propcfg.width = avi_edit_get_width (vmaid->avi_edit[0]);
+      propcfg.height = avi_edit_get_height (vmaid->avi_edit[0]);
+      propcfg.bit_count = avi_edit_get_bit_count (vmaid->avi_edit[0]);
       propcfg.x_pixels_per_meter
-                            = avi_get_x_pixels_per_meter (vmaid->avi_edit[0]);
+                        = avi_edit_get_x_pixels_per_meter (vmaid->avi_edit[0]);
       propcfg.y_pixels_per_meter
-                            = avi_get_y_pixels_per_meter (vmaid->avi_edit[0]);
+                        = avi_edit_get_y_pixels_per_meter (vmaid->avi_edit[0]);
     }
   else
     {
@@ -304,55 +412,69 @@ command_property (GtkWidget *widget,
     }
   if (vmaid->avi_edit[1])
     {
-      propcfg.priority1 = avi_get_priority (vmaid->avi_edit[1]);
-      propcfg.language1 = avi_get_language (vmaid->avi_edit[1]);
-      if (avi_get_name (vmaid->avi_edit[1]))
-        propcfg.name1 = g_strdup (avi_get_name (vmaid->avi_edit[1]));
-      propcfg.samples_per_sec = avi_get_samples_per_sec (vmaid->avi_edit[1]);
-      propcfg.channels = avi_get_channels (vmaid->avi_edit[1]);
-      propcfg.bits_per_sample = avi_get_bits_per_sample (vmaid->avi_edit[1]);
+      propcfg.priority1 = avi_edit_get_priority (vmaid->avi_edit[1]);
+      propcfg.language1 = avi_edit_get_language (vmaid->avi_edit[1]);
+      if (avi_edit_get_name (vmaid->avi_edit[1]))
+        propcfg.name1 = g_strdup (avi_edit_get_name (vmaid->avi_edit[1]));
+      propcfg.samples_per_sec
+                        = avi_edit_get_samples_per_sec (vmaid->avi_edit[1]);
+      propcfg.channels = avi_edit_get_channels (vmaid->avi_edit[1]);
+      propcfg.bits_per_sample
+                        = avi_edit_get_bits_per_sample (vmaid->avi_edit[1]);
     }
   if (property_dialog (&propcfg))
     {
+      gint max, sx;
+    
       vmaid->rate = propcfg.rate;
       vmaid->scale = propcfg.scale;
       if (vmaid->avi_edit[0])
         {
-          avi_set_priority (vmaid->avi_edit[0], propcfg.priority0);
-          avi_set_language (vmaid->avi_edit[0], propcfg.language0);
-          avi_set_name (vmaid->avi_edit[0], propcfg.name0);
-          avi_set_rate (vmaid->avi_edit[0], propcfg.rate);
-          avi_set_scale (vmaid->avi_edit[0], propcfg.scale);
-          avi_set_width (vmaid->avi_edit[0], propcfg.width);
-          avi_set_height (vmaid->avi_edit[0], propcfg.height);
-          avi_set_bit_count (vmaid->avi_edit[0], propcfg.bit_count);
-          avi_set_x_pixels_per_meter (vmaid->avi_edit[0],
+          avi_edit_set_priority (vmaid->avi_edit[0], propcfg.priority0);
+          avi_edit_set_language (vmaid->avi_edit[0], propcfg.language0);
+          avi_edit_set_name (vmaid->avi_edit[0], propcfg.name0);
+          avi_edit_set_rate (vmaid->avi_edit[0], propcfg.rate);
+          avi_edit_set_scale (vmaid->avi_edit[0], propcfg.scale);
+          avi_edit_set_width (vmaid->avi_edit[0], propcfg.width);
+          avi_edit_set_height (vmaid->avi_edit[0], propcfg.height);
+          avi_edit_set_bit_count (vmaid->avi_edit[0], propcfg.bit_count);
+          avi_edit_set_x_pixels_per_meter (vmaid->avi_edit[0],
                                                 propcfg.x_pixels_per_meter);
-          avi_set_y_pixels_per_meter (vmaid->avi_edit[0],
+          avi_edit_set_y_pixels_per_meter (vmaid->avi_edit[0],
                                                 propcfg.y_pixels_per_meter);
           g_free (propcfg.name0);
         }
       if (vmaid->avi_edit[1])
         {
-          avi_set_priority (vmaid->avi_edit[1], propcfg.priority1);
-          avi_set_language (vmaid->avi_edit[1], propcfg.language1);
-          avi_set_name (vmaid->avi_edit[1], propcfg.name1);
-          avi_set_samples_per_sec (vmaid->avi_edit[1],
+          avi_edit_set_priority (vmaid->avi_edit[1], propcfg.priority1);
+          avi_edit_set_language (vmaid->avi_edit[1], propcfg.language1);
+          avi_edit_set_name (vmaid->avi_edit[1], propcfg.name1);
+          avi_edit_set_samples_per_sec (vmaid->avi_edit[1],
                                                     propcfg.samples_per_sec);
-          avi_set_channels (vmaid->avi_edit[1], propcfg.channels);
-          avi_set_bits_per_sample (vmaid->avi_edit[1],
+          avi_edit_set_channels (vmaid->avi_edit[1], propcfg.channels);
+          avi_edit_set_bits_per_sample (vmaid->avi_edit[1],
                                                     propcfg.bits_per_sample);
           g_free (propcfg.name1);
         }
-      vmaid->cursor.stream = vmaid->avi_edit[0] ? 0 : 1;
-      vmaid->cursor.frame = 0;
-      vmaid->select.stream = -1;
-      vmaid->select.frame = -1;
-      vmaid->top = 0;
       delete_list (&vmaid->undo);
       delete_list (&vmaid->redo);
+      max = get_max_frame (vmaid, -1) + 1;
+      sx = get_sx_frame (vmaid);
+      vmaid->select.stream = -1;
+      if (vmaid->cursor.frame > max)
+        vmaid->cursor.frame = max;
+      /* ja:キャレットが表示されるように調整 */
+      if (vmaid->top > MIN (vmaid->cursor.frame, max - sx))
+        vmaid->top = MIN (vmaid->cursor.frame, max - sx);
+      if (vmaid->top < MAX (vmaid->cursor.frame - sx, 0))
+        vmaid->top = MAX (vmaid->cursor.frame - sx, 0);
+      /* ja:メニュー,ステートメント,子ウインドウ */
       set_menu_bar (vmaid);
       gtk_widget_draw (vmaid->drawing, NULL);
+      draw_caret (vmaid, NULL);
+      misc_set_scroll_bar (vmaid->hscroll,
+                                    G_CALLBACK (signal_value_changed_hscroll),
+                                                vmaid, 0, max, sx, vmaid->top);
     }
 }
 
@@ -405,83 +527,40 @@ command_copy (GtkWidget *widget,
   AviEdit *avi_edit[3];
   VmaidWindow *vmaid;
 
-#ifndef G_OS_WIN32
-  if (gtk_selection_owner_set (window, GDK_SELECTION_CLIPBOARD,
-                                                            GDK_CURRENT_TIME))
+  vmaid = orz_mdi_get_data (ORZ_MDI (mdi), -1);
+  /* ja:ビデオ */
+  if (MIN (vmaid->cursor.stream, vmaid->select.stream) == 0)
     {
-#endif /* not G_OS_WIN32 */
-      vmaid = orz_mdi_get_data (ORZ_MDI (mdi), -1);
-      /* ja:ビデオ */
-      if (MIN (vmaid->cursor.stream, vmaid->select.stream) == 0)
-        {
-          max = avi_length (vmaid->avi_edit[0]);
-          start = MAX (MIN (vmaid->cursor.frame, vmaid->select.frame), 0);
-          end = MIN (MAX (vmaid->cursor.frame, vmaid->select.frame), max);
-          length = end - start;
-          if (length > 0 && (avi_edit[i]
-                            = avi_copy (vmaid->avi_edit[0], start, length)))
-            i++;
-        }
-      /* ja:オーディオ */
-      if (MAX (vmaid->cursor.stream, vmaid->select.stream) == 1)
-        {
-          max = avi_length (vmaid->avi_edit[1]);
-          start = MAX (avi_time_to_sample (vmaid->avi_edit[1], (gint)
+      max = avi_edit_length (vmaid->avi_edit[0]);
+      start = MAX (MIN (vmaid->cursor.frame, vmaid->select.frame), 0);
+      end = MIN (MAX (vmaid->cursor.frame, vmaid->select.frame), max);
+      length = end - start;
+      if (length > 0 && (avi_edit[i]
+                        = avi_edit_copy (vmaid->avi_edit[0], start, length)))
+        i++;
+    }
+  /* ja:オーディオ */
+  if (MAX (vmaid->cursor.stream, vmaid->select.stream) == 1)
+    {
+      max = avi_edit_length (vmaid->avi_edit[1]);
+      start = MAX (avi_time_to_sample (vmaid->avi_edit[1], (gint)
             ((glonglong)MIN (vmaid->cursor.frame, vmaid->select.frame) * 1000
                                             * vmaid->scale / vmaid->rate)), 0);
-          end = MIN (avi_time_to_sample (vmaid->avi_edit[1], (gint)
+      end = MIN (avi_time_to_sample (vmaid->avi_edit[1], (gint)
             ((glonglong)MAX (vmaid->cursor.frame, vmaid->select.frame) * 1000
                                         * vmaid->scale / vmaid->rate)), max);
-          length = end - start;
-          if (length > 0 && (avi_edit[i]
-                            = avi_copy (vmaid->avi_edit[1], start, length)))
-            i++;
-        }
-      if (i > 0)
-        {
-          g_free (clipboard_scenario);
-          avi_edit[i] = NULL;
-          clipboard_scenario = avi_to_scenario (avi_edit, FALSE);
-          for (i = 0; avi_edit[i]; i++)
-            avi_release (avi_edit[i]);
-#ifdef G_OS_WIN32
-          if (clipboard_scenario)
-            {
-              HGLOBAL hGlobal;
-              LPSTR lpScenario;
-
-              hGlobal = GlobalAlloc (GMEM_MOVEABLE,
-                        (g_strlen (clipboard_scenario) + 1) * sizeof (gchar));
-              if (hGlobal)
-                {
-                  lpScenario = GlobalLock (hGlobal);
-                  if (lpScenario)
-                    {
-                      g_strcpy (lpScenario, clipboard_scenario);
-                      GlobalUnlock (hGlobal);
-                      if (OpenClipboard (GDK_WINDOW_HWND (window->window)))
-                        {
-                          if (!EmptyClipboard ()
-                                    || !SetClipboardData (uFormat, hGlobal))
-                            GlobalFree (hGlobal);
-                          CloseClipboard ();
-                        }
-                      else
-                        {
-                          GlobalFree (hGlobal);
-                        }
-                    }
-                  else
-                    {
-                      GlobalFree (hGlobal);
-                    }
-                }
-            }
-#endif /* G_OS_WIN32 */
-        }
-#ifndef G_OS_WIN32
+      length = end - start;
+      if (length > 0 && (avi_edit[i]
+                        = avi_edit_copy (vmaid->avi_edit[1], start, length)))
+        i++;
+    }
+  if (i > 0)
+    {
+      avi_edit[i] = NULL;
+      avi_clip_copy (AVI_CLIP (clipboard), avi_edit);
+      for (i = 0; avi_edit[i]; i++)
+        avi_edit_close (avi_edit[i]);
     }
-#endif /* not G_OS_WIN32 */
 }
 
 
@@ -489,38 +568,7 @@ void
 command_paste (GtkWidget *widget,
                gpointer   user_data)
 {
-#ifdef G_OS_WIN32
-  gchar *scenario = NULL;
-  HGLOBAL hGlobal;
-  LPSTR lpScenario;
-
-  if (OpenClipboard (GDK_WINDOW_HWND (window->window)))
-    {
-      if (hGlobal = GetClipboardData (uFormat))
-        {
-          if (lpScenario = GlobalLock (hGlobal))
-            scenario = g_strdup (lpScenario);
-          GlobalUnlock (hGlobal);
-        }
-      CloseClipboard ();
-      if (scenario)
-        {
-          GtkSelectionData data;
-
-          data.selection = GDK_SELECTION_CLIPBOARD;
-          data.target = atom_scenario;
-          data.type = atom_scenario;
-          data.format = 8;
-          data.data = scenario;
-          data.length = g_strlen (scenario);
-          signal_selection_received (window, &data, GDK_CURRENT_TIME, NULL);
-          g_free (scenario);
-        }
-    }
-#else /* not G_OS_WIN32 */
-  gtk_selection_convert (window, GDK_SELECTION_CLIPBOARD, atom_targets,
-                                                            GDK_CURRENT_TIME);
-#endif /* not G_OS_WIN32 */
+  avi_clip_paste (AVI_CLIP (clipboard));
 }
 
 
@@ -538,7 +586,7 @@ command_delete (GtkWidget *widget,
   d = g_malloc0 (sizeof (VmaidHistory));
   for (i = 0; i < 2; i++)
     if (vmaid->avi_edit[i]
-                        && !(d->avi_edit[i] = avi_clone (vmaid->avi_edit[i])))
+                    && !(d->avi_edit[i] = avi_edit_dup (vmaid->avi_edit[i])))
       result = FALSE;
   if (!result)
     {
@@ -548,11 +596,13 @@ command_delete (GtkWidget *widget,
                             GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                             GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
                             _("AVI Clone Error"));
+      g_signal_connect (G_OBJECT (dialog), "key-press-event",
+                                    G_CALLBACK (misc_dialog_key_press), NULL);
       gtk_dialog_run (GTK_DIALOG (dialog));
       gtk_widget_destroy (dialog);
       for (i = 0; i < 2; i++)
         if (d->avi_edit[i])
-          avi_release (d->avi_edit[i]);
+          avi_edit_close (d->avi_edit[i]);
       g_free (d);
       return;
     }
@@ -561,26 +611,27 @@ command_delete (GtkWidget *widget,
   /* ja:ビデオ */
   if (MIN (vmaid->cursor.stream, vmaid->select.stream) == 0)
     {
-      max = avi_length (vmaid->avi_edit[0]);
+      max = avi_edit_length (vmaid->avi_edit[0]);
       start = MAX (MIN (vmaid->cursor.frame, vmaid->select.frame), 0);
       end = MIN (MAX (vmaid->cursor.frame, vmaid->select.frame), max);
       if (start == 0 && end == max)
         {
-          if (!avi_release (vmaid->avi_edit[0]))
+          if (!avi_edit_close (vmaid->avi_edit[0]))
             result = FALSE;
           vmaid->avi_edit[0] = NULL;
         }
       else
         {
           length = end - start;
-          if (length > 0 && !avi_delete(vmaid->avi_edit[0], start, length))
-            result=FALSE;
+          if (length > 0
+                    && !avi_edit_delete (vmaid->avi_edit[0], start, length))
+            result = FALSE;
         }
     }
   /* ja:オーディオ */
   if (MAX (vmaid->cursor.stream, vmaid->select.stream) == 1)
     {
-      max = avi_length (vmaid->avi_edit[1]);
+      max = avi_edit_length (vmaid->avi_edit[1]);
       start = MAX (avi_time_to_sample (vmaid->avi_edit[1], (gint)
             ((glonglong)MIN (vmaid->cursor.frame, vmaid->select.frame) * 1000
                                             * vmaid->scale / vmaid->rate)), 0);
@@ -589,14 +640,15 @@ command_delete (GtkWidget *widget,
                                         * vmaid->scale / vmaid->rate)), max);
       if (start == 0 && end == max)
         {
-          if (!avi_release (vmaid->avi_edit[1]))
+          if (!avi_edit_close (vmaid->avi_edit[1]))
             result=FALSE;
           vmaid->avi_edit[1] = NULL;
         }
       else
         {
           length = end - start;
-          if (length > 0 && !avi_delete (vmaid->avi_edit[1], start, length))
+          if (length > 0
+                    && !avi_edit_delete (vmaid->avi_edit[1], start, length))
             result = FALSE;
         }
     }
@@ -608,12 +660,14 @@ command_delete (GtkWidget *widget,
                             GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                             GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
                             _("AVI Delete Error"));
+      g_signal_connect (G_OBJECT (dialog), "key-press-event",
+                                    G_CALLBACK (misc_dialog_key_press), NULL);
       gtk_dialog_run (GTK_DIALOG (dialog));
       gtk_widget_destroy (dialog);
       for (i = 0; i < 2; i++)
         {
           if (vmaid->avi_edit[i])
-            avi_release (vmaid->avi_edit[i]);
+            avi_edit_close (vmaid->avi_edit[i]);
           vmaid->avi_edit[i] = d->avi_edit[i];
         }
       g_free (d);
@@ -623,7 +677,7 @@ command_delete (GtkWidget *widget,
   vmaid->undo = d;
   delete_list (&vmaid->redo);
   max = get_max_frame (vmaid, -1) + 1;
-  sx = MAX ((vmaid->drawing->allocation.width - 4) / vmaid->width - 2, 1);
+  sx = get_sx_frame (vmaid);
   vmaid->cursor.frame = (vmaid->avi_edit[0] || vmaid->avi_edit[1])
                         ? MIN (vmaid->cursor.frame, vmaid->select.frame) : 0;
   vmaid->cursor.stream = !vmaid->avi_edit[0] && !vmaid->avi_edit[1]
@@ -632,17 +686,13 @@ command_delete (GtkWidget *widget,
   if (vmaid->cursor.frame > max)
     vmaid->cursor.frame = max;
   /* ja:キャレットが表示されるように調整 */
-  if (vmaid->top > vmaid->cursor.frame)
+  if (vmaid->top > MIN (vmaid->cursor.frame, max - sx))
     vmaid->top = MIN (vmaid->cursor.frame, max - sx);
-  if (vmaid->top < vmaid->cursor.frame - sx)
+  if (vmaid->top < MAX (vmaid->cursor.frame - sx, 0))
     vmaid->top = MAX (vmaid->cursor.frame - sx, 0);
   /* ja:メニュー,ステートメント,子ウインドウ */
   set_menu_bar (vmaid);
-  gtk_widget_set_usize (vmaid->drawing, vmaid->width * 3 + 4,
-            system_font_height * 2
-            + ((vmaid->avi_edit[0] ? 1 : 0) + (vmaid->avi_edit[1] ? 1 : 0))
-                                                            * vmaid->height);
-  gtk_widget_draw (vmaid->drawing, NULL);
+  set_usize (vmaid);
   draw_caret (vmaid, NULL);
   misc_set_scroll_bar (vmaid->hscroll,
                                     G_CALLBACK (signal_value_changed_hscroll),
@@ -652,6 +702,88 @@ command_delete (GtkWidget *widget,
 }
 
 
+void
+command_merge (GtkWidget *widget,
+               gpointer   user_data)
+{
+  merge_dialog ();
+}
+
+
+void
+command_change (GtkWidget *widget,
+                gpointer   user_data)
+{
+  gint st = 0, ed = 0;
+  ChangeConfig chgcfg;
+  VmaidWindow *vmaid;
+
+  vmaid = orz_mdi_get_data (ORZ_MDI (mdi), -1);
+  if (vmaid->cursor.stream == 0 && vmaid->select.stream == 0)
+    {
+      st = MIN (vmaid->cursor.frame, vmaid->select.frame);
+      ed = MIN (MAX (vmaid->cursor.frame, vmaid->select.frame),
+                                        avi_edit_length (vmaid->avi_edit[0]));
+    }
+  if (ed - st <= 1)
+    {
+      st = 0;
+      ed = avi_edit_length (vmaid->avi_edit[0]);
+      chgcfg.times = vmaid->avi_edit[1] ? avi_time_length (vmaid->avi_edit[1])
+                                        : 0;
+    }
+  else
+    {
+      chgcfg.times = 0;
+    }
+  chgcfg.frames = ed - st;
+  chgcfg.rate = vmaid->rate;
+  chgcfg.scale = vmaid->scale;
+  if (change_dialog (&chgcfg))
+    {
+      ChangeFrames *chgfrm;
+
+      chgfrm = g_malloc (sizeof (ChangeFrames));
+      chgfrm->frames = chgcfg.frames;
+      chgfrm->avi_edit = avi_edit_copy (vmaid->avi_edit[0], st, ed - st);
+#ifdef USE_THREAD
+      g_thread_create ((GThreadFunc)change_frames, chgfrm, TRUE, NULL);
+#else /* not USE_THREAD */
+      change_frames (chgfrm);
+#endif /* not USE_THREAD */
+    }
+}
+
+
+void
+command_reverse (GtkWidget *widget,
+                 gpointer   user_data)
+{
+  gint st = 0, ed = 0;
+  AviEdit *avi_edit;
+  VmaidWindow *vmaid;
+
+  vmaid = orz_mdi_get_data (ORZ_MDI (mdi), -1);
+  if (vmaid->cursor.stream == 0 || vmaid->select.stream == 0)
+    {
+      st = MIN (vmaid->cursor.frame, vmaid->select.frame);
+      ed = MIN (MAX (vmaid->cursor.frame, vmaid->select.frame),
+                                        avi_edit_length (vmaid->avi_edit[0]));
+    }
+  if (ed - st <= 1)
+    {
+      st = 0;
+      ed = avi_edit_length (vmaid->avi_edit[0]);
+    }
+  avi_edit = avi_edit_copy (vmaid->avi_edit[0], st, ed - st);
+#ifdef USE_THREAD
+  g_thread_create ((GThreadFunc)reverse_frame, avi_edit, TRUE, NULL);
+#else /* not USE_THREAD */
+  reverse_frame (avi_edit);
+#endif /* not USE_THREAD */
+}
+
+
 void
 command_jump (GtkWidget *widget,
               gpointer   user_data)
@@ -665,29 +797,30 @@ command_jump (GtkWidget *widget,
   cursor = vmaid->cursor;
   select = vmaid->select;
   top = vmaid->top;
-  jmpcfg.frame = vmaid->cursor.frame;
+  jmpcfg.video = vmaid->avi_edit[0] != NULL;
+  jmpcfg.audio = vmaid->avi_edit[1] != NULL;
   jmpcfg.max = get_max_frame (vmaid, -1) + 1;
   jmpcfg.rate = vmaid->rate;
   jmpcfg.scale = vmaid->scale;
+  jmpcfg.cursor = vmaid->cursor;
+  jmpcfg.select = vmaid->select;
   if (jump_dialog (&jmpcfg))
     {
-      vmaid->cursor.frame = jmpcfg.frame;
-      sx = MAX ((vmaid->drawing->allocation.width - 4) / vmaid->width - 2, 1);
+      vmaid->cursor = jmpcfg.cursor;
+      vmaid->select = jmpcfg.select;
+      sx = get_sx_frame (vmaid);
       /* ja:キャレットが表示されるように調整 */
-      if (vmaid->top > vmaid->cursor.frame)
+      if (vmaid->top > MIN (vmaid->cursor.frame, jmpcfg.max - sx))
         vmaid->top = MIN (vmaid->cursor.frame, jmpcfg.max - sx);
-      if (vmaid->top < vmaid->cursor.frame - sx)
+      if (vmaid->top < MAX (vmaid->cursor.frame - sx, 0))
         vmaid->top = MAX (vmaid->cursor.frame - sx, 0);
       /* ja:ジャンプの例外 ジャンプ先のフレームを表示する */
       if (vmaid->cursor.frame < jmpcfg.max
                                     && vmaid->top + sx <= vmaid->cursor.frame)
         vmaid->top = vmaid->cursor.frame - sx + 1;
-      if (vmaid->cursor.frame == vmaid->select.frame)
-        {
-          /* ja:選択を解除する */
-          vmaid->select.stream = -1;
-          set_menu_bar (vmaid);
-        }
+      if ((vmaid->select.stream >= 0 && select.stream < 0)
+                        || (vmaid->select.stream < 0 && select.stream >= 0))
+        set_menu_bar (vmaid);
       if (vmaid->top != top)
         misc_set_scroll_bar (vmaid->hscroll,
                                     G_CALLBACK (signal_value_changed_hscroll),
@@ -712,18 +845,18 @@ command_all (GtkWidget *widget,
       cursor = vmaid->cursor;
       select = vmaid->select;
       top = vmaid->top;
-      sx = MAX ((vmaid->drawing->allocation.width - 4) / vmaid->width - 2, 1);
+      sx = get_sx_frame (vmaid);
       max = get_max_frame (vmaid, -1) + 1;
       vmaid->select.frame = 0;
       vmaid->cursor.frame = max;
       vmaid->select.stream = vmaid->avi_edit[0] ? 0 : 1;
       vmaid->cursor.stream = vmaid->avi_edit[1] ? 1 : 0;
       /* ja:キャレットが表示されるように調整 */
-      if (vmaid->top > vmaid->cursor.frame)
+      if (vmaid->top > MIN (vmaid->cursor.frame, max - sx))
         vmaid->top = MIN (vmaid->cursor.frame, max - sx);
-      if (vmaid->top < vmaid->cursor.frame - sx)
+      if (vmaid->top < MAX (vmaid->cursor.frame - sx, 0))
         vmaid->top = MAX (vmaid->cursor.frame - sx, 0);
-      if (select.stream< 0 )
+      if (select.stream < 0)
         set_menu_bar (vmaid);
       if (vmaid->top != top)
         misc_set_scroll_bar (vmaid->hscroll,
@@ -749,8 +882,8 @@ command_size (GtkWidget *widget,
   sizecfg.height = vmaid->height;
   if (vmaid->avi_edit[0])
     {
-      sizecfg.def_width = avi_get_width (vmaid->avi_edit[0]);
-      sizecfg.def_height = avi_get_height (vmaid->avi_edit[0]);
+      sizecfg.def_width = avi_edit_get_width (vmaid->avi_edit[0]);
+      sizecfg.def_height = avi_edit_get_height (vmaid->avi_edit[0]);
     }
   else
     {
@@ -763,15 +896,25 @@ command_size (GtkWidget *widget,
       default_ratio = sizecfg.ratio;
       vmaid->width = sizecfg.width;
       vmaid->height = sizecfg.height;
-      gtk_widget_set_usize (vmaid->drawing, vmaid->width * 3 + 4,
-                        system_font_height * 2 + ((vmaid->avi_edit[0] ? 1 : 0)
-                            + (vmaid->avi_edit[1] ? 1 : 0)) * vmaid->height);
-      gtk_widget_queue_resize (GTK_WIDGET (window));
-      gtk_widget_draw (vmaid->drawing, NULL);
+      set_usize (vmaid);
     }
 }
 
 
+void
+command_frame (GtkWidget *widget,
+               gpointer   user_data)
+{
+  VmaidWindow *vmaid;
+
+  vmaid = orz_mdi_get_data (ORZ_MDI (mdi), -1);
+  vmaid->selfrm = gtk_check_menu_item_get_active
+        (GTK_CHECK_MENU_ITEM (misc_find_menu (menu_entries, "/view/frame")));
+  set_usize (vmaid);
+  default_selfrm = vmaid->selfrm;
+}
+
+
 void
 command_conf (GtkWidget *widget,
               gpointer   user_data)
