@@ -1,6 +1,6 @@
 /*
     Video maid
-    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2006 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,11 +17,11 @@
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #include "command.h"
+#include "file.h"
 #include "general.h"
 #include "sigfile.h"
+#include "size.h"
 #include "thread.h"
-#include "toolbar.h"
-#include "misc/misc.h"
 #include "orz/orzmdi.h"
 
 
@@ -43,11 +43,12 @@ hide_caret (VmaidWindow *vmaid)
   if (caret)
     {
       caret = FALSE;
-      sx = MAX ((vmaid->drawing->allocation.width - 4) / vmaid->width - 2, 1);
+      sx = get_sx_frame (vmaid);
       if (vmaid->top <= vmaid->cursor.frame
                                     && vmaid->cursor.frame <= vmaid->top + sx)
         {
-          rc.x = (vmaid->cursor.frame - vmaid->top + 1) * vmaid->width + 1;
+          rc.x = (vmaid->cursor.frame - vmaid->top) * vmaid->width
+                                    + (vmaid->selfrm ? vmaid->width + 1 : 0);
           rc.y = system_font_height * 2
                         + (vmaid->avi_edit[0] && vmaid->cursor.stream > 0
                                                         ? vmaid->height : 0);
@@ -71,7 +72,7 @@ draw_caret (VmaidWindow *vmaid,
   guint context_id;
   GdkRectangle rc;
 
-  sx = MAX ((vmaid->drawing->allocation.width - 4) / vmaid->width - 2, 1);
+  sx = get_sx_frame (vmaid);
   if (cursor && timer_id != 0 && (vmaid->cursor.frame != cursor->frame
                                     || vmaid->cursor.stream != cursor->stream))
     {
@@ -81,7 +82,8 @@ draw_caret (VmaidWindow *vmaid,
       /* ja:古いキャレットを消去 */
       if (vmaid->top <= cursor->frame && cursor->frame <= vmaid->top + sx)
         {
-          rc.x = (cursor->frame-vmaid->top + 1) * vmaid->width + 1;
+          rc.x = (cursor->frame - vmaid->top) * vmaid->width
+                                    + (vmaid->selfrm ? vmaid->width + 1 : 0);
           rc.y = system_font_height * 2
                                     + (vmaid->avi_edit[0] && cursor->stream > 0
                                                         ? vmaid->height : 0);
@@ -96,7 +98,8 @@ draw_caret (VmaidWindow *vmaid,
   if (vmaid->top <= vmaid->cursor.frame
                                     && vmaid->cursor.frame <= vmaid->top + sx)
     {
-      rc.x = (vmaid->cursor.frame - vmaid->top + 1) * vmaid->width + 1;
+      rc.x = (vmaid->cursor.frame - vmaid->top) * vmaid->width
+                                    + (vmaid->selfrm ? vmaid->width + 1 : 0);
       rc.y = system_font_height * 2
                             + (vmaid->avi_edit[0] && vmaid->cursor.stream > 0
                                                         ? vmaid->height : 0);
@@ -150,11 +153,18 @@ set_menu_bar (VmaidWindow *vmaid)
                 "/edit/copy"), vmaid->select.stream >= 0);
       gtk_widget_set_sensitive (misc_find_menu (menu_entries,
                 "/edit/delete"), vmaid->select.stream >= 0);
+      gtk_widget_set_sensitive (misc_find_menu (menu_entries,
+                "/edit/change"), vmaid->avi_edit[0] != NULL);
+      gtk_widget_set_sensitive (misc_find_menu (menu_entries,
+                "/edit/reverse"), vmaid->avi_edit[0]
+                                && avi_edit_length (vmaid->avi_edit[0]) > 1);
       gtk_widget_set_sensitive (misc_find_menu (menu_entries,
                 "/edit/jump"), vmaid->avi_edit[0] || vmaid->avi_edit[1]);
       gtk_widget_set_sensitive (misc_find_menu (menu_entries,
                 "/edit/all"), vmaid->avi_edit[0] || vmaid->avi_edit[1]);
       gtk_widget_set_sensitive (misc_find_menu (menu_entries, "/view"), TRUE);
+      gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (misc_find_menu
+                                (menu_entries, "/view/frame")), vmaid->selfrm);
       gtk_widget_set_sensitive (GTK_WIDGET (misc_find_toolbar (toolbar_entries,
                                         "saveas")), TRUE);
       gtk_widget_set_sensitive (GTK_WIDGET (misc_find_toolbar (toolbar_entries,
@@ -226,17 +236,17 @@ clear_sel (VmaidWindow *vmaid,
   GdkGC *gc;
   GdkRectangle rc, rc0, rc1;
 
-  sx = MAX ((vmaid->drawing->allocation.width - 4) / vmaid->width - 2, 1);
+  sx = get_sx_frame (vmaid);
   /* ja:スクロール */
   if (vmaid->top != top)
     {
       /* ja:移動量が多すぎるときには全体を再描画 */
-      rc.x = vmaid->width + 2;
+      rc.x = vmaid->selfrm ? vmaid->width + 2 : 1;
       rc.y = 0;
       rc.width = vmaid->width * sx;
       rc.height = system_font_height * 2 + ((vmaid->avi_edit[0] ? 1 : 0)
                             + (vmaid->avi_edit[1] ? 1 : 0)) * vmaid->height;
-      if (ABS (top-vmaid->top) <= sx)
+      if (ABS (top - vmaid->top) < sx)
         {
           /* ja:移動量が小さいとき */
           dx = (top - vmaid->top) * vmaid->width;
@@ -256,9 +266,10 @@ clear_sel (VmaidWindow *vmaid,
             || vmaid->cursor.frame == top || vmaid->cursor.frame == top + sx)
         {
           /* ja:端のキャレット */
-          rc0.x = vmaid->cursor.frame == vmaid->top
+          rc0.x = (vmaid->cursor.frame == vmaid->top
                                                 || vmaid->cursor.frame == top
-                            ? vmaid->width + 1 : vmaid->width * (sx + 1) + 2;
+                                                ? 0 : vmaid->width * sx + 1)
+                                    + (vmaid->selfrm ? vmaid->width + 1 : 0);
           rc0.y = system_font_height * 2
                             + (vmaid->avi_edit[0] && vmaid->cursor.stream > 0
                                                         ? vmaid->height : 0);
@@ -271,8 +282,9 @@ clear_sel (VmaidWindow *vmaid,
                         || cursor->frame == top || cursor->frame == top + sx)
         {
           /* ja:端のキャレット */
-          rc0.x = cursor->frame == vmaid->top || cursor->frame == top
-                            ? vmaid->width + 1 : vmaid->width * (sx + 1) + 2;
+          rc0.x = (cursor->frame == vmaid->top || cursor->frame == top
+                                                ? 0 : vmaid->width * sx + 1)
+                                    + (vmaid->selfrm ? vmaid->width + 1 : 0);
           rc0.y = system_font_height * 2
                                     + (vmaid->avi_edit[0] && cursor->stream > 0
                                                         ? vmaid->height : 0);
@@ -286,58 +298,61 @@ clear_sel (VmaidWindow *vmaid,
     {
       rc.x = -1;
     }
-  /* ja:以前の左右の選択について */
-  if (select->stream >= 0)
+  if (vmaid->selfrm)
     {
-      frame = select->frame - (cursor->frame < select->frame ? 1 : 0);
-      if (frame < top)
-        left_old = frame;
-      else if (top + sx <= frame)
-        right_old = frame;
-    }
-  /* ja:現在の左右の選択について */
-  if (vmaid->select.stream >= 0)
-    {
-      frame = vmaid->select.frame
+      /* ja:以前の左右の選択について */
+      if (select->stream >= 0)
+        {
+          frame = select->frame - (cursor->frame < select->frame ? 1 : 0);
+          if (frame < top)
+            left_old = frame;
+          else if (top + sx <= frame)
+            right_old = frame;
+        }
+      /* ja:現在の左右の選択について */
+      if (vmaid->select.stream >= 0)
+        {
+          frame = vmaid->select.frame
                         - (vmaid->cursor.frame < vmaid->select.frame ? 1 : 0);
-      if (frame < vmaid->top)
-        left_new = frame;
-      else if (vmaid->top + sx <= frame)
-        right_new = frame;
-    }
-  if (left_old != left_new || right_old != right_new)
-    {
-      rc0.y = 0;
-      rc0.width = vmaid->width;
-      rc0.height
+          if (frame < vmaid->top)
+            left_new = frame;
+          else if (vmaid->top + sx <= frame)
+            right_new = frame;
+        }
+      if (left_old != left_new || right_old != right_new)
+        {
+          rc0.y = 0;
+          rc0.width = vmaid->width;
+          rc0.height
                 = ((vmaid->avi_edit[0] ? 1 : 0) + (vmaid->avi_edit[1] ? 1 : 0))
                                     * vmaid->height + system_font_height * 2;
-      /* ja:左側について */
-      if (left_old != left_new)
-        {
-          rc0.x = 0;
-          if (rc.x < 0)
+          /* ja:左側について */
+          if (left_old != left_new)
             {
-              rc = rc0;
+              rc0.x = 0;
+              if (rc.x < 0)
+                {
+                  rc = rc0;
+                }
+              else
+                {
+                  rc1 = rc;
+                  gdk_rectangle_union (&rc0, &rc1, &rc);
+                }
             }
-          else
+          /* ja:右側について */
+          if (right_old != right_new)
             {
-              rc1 = rc;
-              gdk_rectangle_union (&rc0, &rc1, &rc);
-            }
-        }
-      /* ja:右側について */
-      if (right_old != right_new)
-        {
-          rc0.x = (sx + 1) * vmaid->width + 4;
-          if (rc.x < 0)
-            {
-              rc = rc0;
-            }
-          else
-            {
-              rc1 = rc;
-              gdk_rectangle_union (&rc0, &rc1, &rc);
+              rc0.x = (sx + 1) * vmaid->width + 4;
+              if (rc.x < 0)
+                {
+                  rc = rc0;
+                }
+              else
+                {
+                  rc1 = rc;
+                  gdk_rectangle_union (&rc0, &rc1, &rc);
+                }
             }
         }
     }
@@ -348,16 +363,15 @@ clear_sel (VmaidWindow *vmaid,
                         - MAX (MIN (cursor->frame, select->frame), vmaid->top);
       if (width > 0)
         {
-          rc0.x
-            = (MAX (MIN (cursor->frame, select->frame) - vmaid->top, 0) + 1)
-                                                            * vmaid->width + 2;
+          rc0.x = MAX (MIN (cursor->frame, select->frame) - vmaid->top, 0)
+                    * vmaid->width + (vmaid->selfrm ? vmaid->width + 2 : 1);
           rc0.y = MIN (cursor->stream, select->stream) * vmaid->height
                                                     + system_font_height * 2;
           rc0.width = width * vmaid->width;
           rc0.height = (ABS (cursor->stream - select->stream) + 1)
                                                             * vmaid->height;
           if (!vmaid->avi_edit[0])
-            rc0.y-=vmaid->height;
+            rc0.y -= vmaid->height;
           gtk_widget_draw (vmaid->drawing, &rc);
           if (rc.x < 0)
             {
@@ -379,8 +393,8 @@ clear_sel (VmaidWindow *vmaid,
                         - MAX (MIN (cursor->frame, select->frame), vmaid->top);
           if (width > 0)
             {
-              rc0.x = (MAX (MIN (cursor->frame, select->frame) - vmaid->top, 0)
-                                                    + 1) * vmaid->width + 2;
+              rc0.x = MAX (MIN (cursor->frame, select->frame) - vmaid->top, 0)
+                    * vmaid->width + (vmaid->selfrm ? vmaid->width + 2 : 1);
               rc0.y = (vmaid->select.stream == 0 ? vmaid->height : 0)
                                                     + system_font_height * 2;
               rc0.width = width * vmaid->width;
@@ -404,8 +418,9 @@ clear_sel (VmaidWindow *vmaid,
                 - MAX (MIN (vmaid->cursor.frame, cursor->frame), vmaid->top);
           if (width > 0)
             {
-              rc0.x = (MAX (MIN (vmaid->cursor.frame, cursor->frame)
-                                    - vmaid->top, 0) + 1) * vmaid->width + 2;
+              rc0.x = MAX (MIN (vmaid->cursor.frame, cursor->frame)
+                                                - vmaid->top, 0) * vmaid->width
+                                    + (vmaid->selfrm ? vmaid->width + 2 : 1);
               rc0.y = MIN (vmaid->cursor.stream, vmaid->select.stream)
                                     * vmaid->height + system_font_height * 2;
               rc0.width = width * vmaid->width;
@@ -430,6 +445,20 @@ clear_sel (VmaidWindow *vmaid,
 }
 
 
+/*  ja:最小のウインドウサイズを設定する
+     vmaid,ウインドウ情報                                                   */
+void
+set_usize (VmaidWindow *vmaid)
+{
+  gtk_widget_set_size_request (vmaid->drawing,
+                    vmaid->width + (vmaid->selfrm ? vmaid->width * 2 + 4 : 2),
+                        system_font_height * 2 + ((vmaid->avi_edit[0] ? 1 : 0)
+                            + (vmaid->avi_edit[1] ? 1 : 0)) * vmaid->height);
+  gtk_widget_queue_resize (GTK_WIDGET (window));
+  gtk_widget_draw (vmaid->drawing, NULL);
+}
+
+
 /*  ja:リストから削除する
       d,リストの先頭
     RET,削除したリストの数                                                  */
@@ -443,7 +472,7 @@ delete_list (VmaidHistory **d)
     {
       for (i = 0; i < 2; i++)
         if (d0->avi_edit[i])
-          avi_release (d0->avi_edit[i]);
+          avi_edit_close (d0->avi_edit[i]);
       d1 = d0->next;
       g_free (d0);
       count++;
@@ -493,20 +522,16 @@ history_operation (VmaidWindow    *vmaid,
       vmaid->redo = d;
     }
   max = get_max_frame (vmaid, -1) + 1;
-  sx = MAX ((vmaid->drawing->allocation.width - 4) / vmaid->width - 2, 1);
+  sx = get_sx_frame (vmaid);
   if (vmaid->cursor.frame > max)
     vmaid->cursor.frame = max;
    /* ja:キャレットが表示されるように調整 */
-  if (vmaid->top > vmaid->cursor.frame)
+  if (vmaid->top > MIN (vmaid->cursor.frame, max - sx))
     vmaid->top = MIN (vmaid->cursor.frame, max - sx);
-  if (vmaid->top < vmaid->cursor.frame - sx)
+  if (vmaid->top < MAX (vmaid->cursor.frame - sx, 0))
     vmaid->top = MAX (vmaid->cursor.frame - sx, 0);
   set_menu_bar (vmaid);
-  gtk_widget_set_usize (vmaid->drawing, vmaid->width * 3 + 4,
-                        system_font_height * 2 + ((vmaid->avi_edit[0] ? 1 : 0)
-                            + (vmaid->avi_edit[1] ? 1 : 0)) * vmaid->height);
-  gtk_widget_queue_resize (GTK_WIDGET (window));
-  gtk_widget_draw (vmaid->drawing, NULL);
+  set_usize (vmaid);
   misc_set_scroll_bar (vmaid->hscroll,
                                 GTK_SIGNAL_FUNC (signal_value_changed_hscroll),
                                                 vmaid, 0, max, sx, vmaid->top);
@@ -529,18 +554,31 @@ prompt_close (VmaidWindow *vmaid)
       gint result;
       GtkWidget *dialog;
 
+#if GLIB_CHECK_VERSION(2,6,0)
+      utf8str = g_filename_display_name
+                                    (orz_mdi_get_file (ORZ_MDI (mdi), page));
+#else /* not GLIB_CHECK_VERSION(2,6,0) */
       utf8str = g_filename_to_utf8
                 (orz_mdi_get_file (ORZ_MDI (mdi), page), -1, NULL, NULL, NULL);
+#endif /* not GLIB_CHECK_VERSION(2,6,0) */
       dialog = gtk_message_dialog_new (GTK_WINDOW (window),
                             GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                             GTK_MESSAGE_INFO, GTK_BUTTONS_NONE,
                             _("File %s was edited.\nSave?"), utf8str);
       g_free (utf8str);
       gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+#ifdef G_OS_WIN32
                                         GTK_STOCK_YES, GTK_RESPONSE_YES,
                                         GTK_STOCK_NO, GTK_RESPONSE_NO,
                                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+#else /* not G_OS_WIN32 */
+                                        GTK_STOCK_NO, GTK_RESPONSE_NO,
+                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                        GTK_STOCK_YES, GTK_RESPONSE_YES,
+#endif /* not G_OS_WIN32 */
                                         NULL);
+      g_signal_connect (G_OBJECT (dialog), "key-press-event",
+                                    G_CALLBACK (misc_dialog_key_press), NULL);
       result = gtk_dialog_run (GTK_DIALOG (dialog));
       gtk_widget_destroy (dialog);
       switch (result)
@@ -571,23 +609,20 @@ prompt_close (VmaidWindow *vmaid)
 }
 
 
-/******************************************************************************
-*                                                                             *
-* ja:高レベル関数群                                                           *
-*                                                                             *
-******************************************************************************/
 /*  ja:最大フレーム数を求める
      vmaid,ウインドウ情報
     stream,有効なストリーム
        RET,最大フレーム数                                                   */
-gint get_max_frame (VmaidWindow *vmaid,
-                    const gint   stream)
+gint
+get_max_frame (VmaidWindow *vmaid,
+               const gint   stream)
 {
   gint max0, max1;
 
-  max0 = vmaid->avi_edit[0] ? avi_length (vmaid->avi_edit[0]) - 1 : 0;
-  max1 = vmaid->avi_edit[1] ? (glonglong)avi_length_time (vmaid->avi_edit[1])
-                    * vmaid->rate / ((glonglong)vmaid->scale * 1000) - 1 : 0;
+  max0 = vmaid->avi_edit[0] ? avi_edit_length (vmaid->avi_edit[0]) - 1 : 0;
+  max1 = vmaid->avi_edit[1] ? ((glonglong)avi_time_length (vmaid->avi_edit[1])
+                                    * vmaid->rate + vmaid->scale * 1000 - 1)
+                                    / ((glonglong)vmaid->scale * 1000) - 1 : 0;
   switch (stream)
     {
       case 0: return max0;
@@ -595,3 +630,96 @@ gint get_max_frame (VmaidWindow *vmaid,
     }
   return MAX (max0, max1);
 }
+
+
+/******************************************************************************
+*                                                                             *
+* ja:高レベル関数群                                                           *
+*                                                                             *
+******************************************************************************/
+/*  ja:フレームを反転する(独立スレッド)
+    avi_edit,AVI編集構造体
+         RET,TRUE:正常終了,FALSE:エラー                                     */
+gboolean
+reverse_frame (AviEdit *avi_edit)
+{
+  gint i;
+  VmaidWindow *vmaid;
+#ifdef USE_THREAD
+  GThread *id;
+#else /* not USE_THREAD */
+  gint id = THREAD_ID_STANDARD;
+#endif /* not USE_THREAD */
+
+#ifdef USE_THREAD
+  id = g_thread_self ();
+#endif /* USE_THREAD */
+  thread_insert (id, THREAD_MODE_REVERSE, NULL);
+
+  vmaid = g_malloc0 (sizeof (VmaidWindow));
+  for (i = 0; i < avi_edit_length (avi_edit); i++)
+    {
+      AviEdit *avi_edit_tmp;
+
+      if (!thread_idling (id, i * 100 / avi_edit_length (avi_edit)))
+        {
+          avi_edit_close (vmaid->avi_edit[0]);
+          vmaid->avi_edit[0] = NULL;
+          break;
+        }
+      avi_edit_tmp = avi_edit_copy (avi_edit, i, 1);
+      if (!avi_edit_tmp)
+        {
+          avi_edit_close (vmaid->avi_edit[0]);
+          vmaid->avi_edit[0] = NULL;
+          while (thread_idling (id, -1));
+          break;
+        }
+      if (vmaid->avi_edit[0])
+        {
+          if (!avi_edit_paste (vmaid->avi_edit[0], 0, avi_edit_tmp))
+            {
+              avi_edit_close (avi_edit_tmp);
+              avi_edit_close (vmaid->avi_edit[0]);
+              vmaid->avi_edit[0] = NULL;
+              while (thread_idling (id, -1));
+              break;
+            }
+        }
+      else
+        {
+          vmaid->avi_edit[0] = avi_edit_tmp;
+        }
+    }
+  if (!avi_edit_close (avi_edit))
+    {
+      avi_edit_close (vmaid->avi_edit[0]);
+      vmaid->avi_edit[0] = NULL;
+      while (thread_idling (id, -1));
+    }
+  if (vmaid->avi_edit[0])
+    {
+      vmaid->selfrm = default_selfrm;
+      vmaid->select.stream = -1;
+      vmaid->select.frame = -1;
+      size_set_scale (default_view, &vmaid->width, &vmaid->height,
+                                    avi_edit_get_width (vmaid->avi_edit[0]),
+                                    avi_edit_get_height (vmaid->avi_edit[0]));
+      vmaid->rate = avi_edit_get_rate (vmaid->avi_edit[0]);
+      vmaid->scale = avi_edit_get_scale (vmaid->avi_edit[0]);
+#ifdef USE_THREAD
+      gdk_threads_enter ();
+#endif /* USE_THREAD */
+      file_open_window (vmaid, NULL);
+#ifdef USE_THREAD
+      gdk_threads_leave ();
+#endif /* USE_THREAD */
+    }
+  else
+    {
+      g_free (vmaid);
+      vmaid = NULL;
+    }
+  thread_delete (id);
+  return vmaid != NULL;
+}
