@@ -1,6 +1,6 @@
 /*
     Video maid
-    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2006 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
 #include "file.h"
 #include "general.h"
 #include "sigfile.h"
-#include "misc/misc.h"
 #include "orz/orzmdi.h"
 
 
@@ -35,7 +34,7 @@ signal_timeout (gpointer data)
 {
   gint page;
 
-  caret =! caret;
+  caret = !caret;
 #ifdef USE_THREAD
   gdk_threads_enter ();
 #endif /* USE_THREAD */
@@ -46,11 +45,12 @@ signal_timeout (gpointer data)
       VmaidWindow *vmaid;
 
       vmaid = orz_mdi_get_data (ORZ_MDI (mdi), page);
-      sx = MAX ((vmaid->drawing->allocation.width - 4) / vmaid->width - 2, 1);
+      sx = get_sx_frame (vmaid);
       if (vmaid->top <= vmaid->cursor.frame
                                     && vmaid->cursor.frame <= vmaid->top + sx)
         {
-          rc.x = (vmaid->cursor.frame - vmaid->top + 1) * vmaid->width + 1;
+          rc.x = (vmaid->cursor.frame - vmaid->top) * vmaid->width
+                                    + (vmaid->selfrm ? vmaid->width + 1 : 0);
           rc.y = system_font_height * 2
                             + (vmaid->avi_edit[0] && vmaid->cursor.stream > 0
                                                         ? vmaid->height : 0);
@@ -110,9 +110,9 @@ signal_value_changed_hscroll (GtkAdjustment *adjust,
 
   top = vmaid->top;
   vmaid->top = adjust->value + 0.5;
-  sx = MAX ((vmaid->drawing->allocation.width - 4) / vmaid->width - 2, 1);
+  sx = get_sx_frame (vmaid);
   max = get_max_frame (vmaid, -1) + 1;
-  if (vmaid->top>max - sx)
+  if (vmaid->top > MAX (max - sx, 0))
     vmaid->top = MAX (max - sx, 0);
   clear_sel (vmaid, &vmaid->cursor, &vmaid->select, top);
 }
@@ -126,8 +126,9 @@ signal_config (GtkWidget         *widget,
   gint sx, max;
 
   max = get_max_frame (vmaid, -1) + 1;
-  sx = MAX ((config->width - 4) / vmaid->width - 2, 1);
-  if (vmaid->top > max - sx)
+  sx = MAX (vmaid->selfrm ? (config->width - 4) / vmaid->width - 2
+                          : (config->width - 2) / vmaid->width, 1);
+  if (vmaid->top > MAX (max - sx, 0))
     vmaid->top = MAX (max - sx, 0);
   draw_caret (vmaid, NULL);
   misc_set_scroll_bar (vmaid->hscroll,
@@ -158,7 +159,7 @@ signal_destroy_draw (GtkWidget   *widget,
     }
   for (i = 0; i < 2; i++)
     if (vmaid->avi_edit[i])
-      avi_release (vmaid->avi_edit[i]);
+      avi_edit_close (vmaid->avi_edit[i]);
   delete_list (&vmaid->undo);
   delete_list (&vmaid->redo);
   g_free (vmaid);
@@ -182,7 +183,7 @@ signal_expose_text (GtkWidget   *widget,
   gint t;
 
   text = g_strdup_printf (vmaid->avi_edit[0]
-            && avi_is_keyframe (vmaid->avi_edit[0],pos) ? "%d*" : "%d", pos);
+            && avi_frame_is_key (vmaid->avi_edit[0],pos) ? "%d*" : "%d", pos);
   if (gdk_string_width (system_font,text) <= vmaid->width)
     gdk_draw_string (widget->window, system_font, gc,
                                                 x, system_font_ascent, text);
@@ -191,7 +192,7 @@ signal_expose_text (GtkWidget   *widget,
   text = g_strdup_printf ("%02d:%02d:%02d",
                                 t / 60000 % 100, t / 1000 % 60, t / 10 % 100);
   if (gdk_string_width (system_font, text) <= vmaid->width)
-    gdk_draw_string (widget->window, system_font,gc,
+    gdk_draw_string (widget->window, system_font, gc,
                         x, y + system_font_ascent + system_font_height, text);
   g_free (text);
 }
@@ -209,7 +210,7 @@ signal_expose_bitmap (GtkWidget *widget,
 {
   guchar *buf;
 
-  buf = avi_get_frame32 (avi_frame, pos, width, height);
+  buf = avi_frame_get_raw (avi_frame, pos, width, height);
   if (buf)
     gdk_draw_rgb_32_image (widget->window, gc, x, y, width, height,
                                         GDK_RGB_DITHER_NORMAL, buf, width * 4);
@@ -231,20 +232,22 @@ signal_expose_wave (GtkWidget  *widget,
                     const gint  height)
 {
   gpointer out_buf = NULL;
-  gint start, end, count, out_samples = 0;
+  gint start, end, count;
+  gsize out_samples = 0;
 
   start = 0;
-  end = avi_length_time (avi_edit);
+  end = avi_time_length (avi_edit);
   if (avi_pcm)
     {
-      gint pos, samples;
+      gint pos;
+      gsize samples;
 
       /* ja:範囲内のPCMを読み込む */
       pos = avi_time_to_sample (avi_edit, MAX (start, current_time));
       samples = avi_time_to_sample (avi_edit, MIN (end, next_time)) - pos;
-      if (samples > avi_length (avi_edit) - pos)
-        samples = avi_length (avi_edit) - pos;
-      avi_get_pcm (avi_pcm, pos, samples, &out_buf, &out_samples);
+      if (samples > avi_edit_length (avi_edit) - pos)
+        samples = avi_edit_length (avi_edit) - pos;
+      avi_pcm_get_raw (avi_pcm, pos, samples, &out_buf, &out_samples);
     }
   if (out_samples <= 0)
     {
@@ -312,7 +315,7 @@ signal_expose_wave (GtkWidget  *widget,
       gint i, x0, x1, y0, y1;
 
       /* ja:波形表示 */
-      gdk_gc_set_foreground (gc, system_color);
+      gdk_gc_set_foreground (gc, system_color + 2);
       x0 = MAX (width * (start - current_time)
                                 / (next_time - current_time) + x, x);
       x1 = MIN (width * (end - current_time)
@@ -324,12 +327,12 @@ signal_expose_wave (GtkWidget  *widget,
 
           /* ja:音量 */
           j = i * out_samples / count;
-          if (avi_get_bits_per_sample (avi_edit) == 8)
+          if (avi_edit_get_bits_per_sample (avi_edit) == 8)
             {
               guint8 *p;
 
               p = out_buf;
-              if (avi_get_channels (avi_edit) == 1)
+              if (avi_edit_get_channels (avi_edit) == 1)
                 {
                   y0 = y1 = ABS (p[j] - 128) * height / 256;
                 }
@@ -345,7 +348,7 @@ signal_expose_wave (GtkWidget  *widget,
               gint16 *p;
 
               p = out_buf;
-              if (avi_get_channels (avi_edit) == 1)
+              if (avi_edit_get_channels (avi_edit) == 1)
                 {
                   y0 = y1 = ABS (GINT16_FROM_LE (p[j])) * height / 65536;
                 }
@@ -356,7 +359,6 @@ signal_expose_wave (GtkWidget  *widget,
                   y1 = ABS (GINT16_FROM_LE (p[j + 1])) * height / 65536;
                 }
             }
-          gdk_gc_set_foreground (gc, system_color + 2);
           gdk_draw_line (widget->window, gc,
                     x0 + i, y + height / 2 - y0, x0 + i, y + height / 2 + y1);
         }
@@ -382,27 +384,25 @@ signal_expose (GtkWidget      *widget,
   gdk_gc_set_line_attributes (gc, 1,
                                 GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_MITER);
   gdk_gc_set_stipple (gc, bitmap0);
-  for (i = 0; i < 4; i++)
-    gdk_color_alloc (gdk_colormap_get_system (), system_color + i);
   gdk_gc_set_clip_rectangle (gc, &event->area);
   /* ja:フレーム数、時間の背景 */
   if (system_font_height * 2 - event->area.y > 0)
     {
-      gdk_gc_set_foreground (gc,system_color + 1);
+      gdk_gc_set_foreground (gc, system_color + 1);
       gdk_draw_rectangle (widget->window, gc, TRUE,
                     event->area.x, event->area.y,
                     event->area.width, system_font_height * 2 - event->area.y);
     }
   /* ja:フレーム数 */
-  sx = MAX ((widget->allocation.width - 4) / vmaid->width - 2, 1);
+  sx = get_sx_frame (vmaid);
   max = MIN (get_max_frame (vmaid, -1), vmaid->top + sx - 1);
-  frame = vmaid->select.stream >= 0 ? vmaid->select.frame
-                - (vmaid->cursor. frame < vmaid->select.frame ? 1 : 0) : -1;
   gdk_gc_set_foreground (gc, system_color);
+  frame = vmaid->selfrm && vmaid->select.stream >= 0 ? vmaid->select.frame
+                    - (vmaid->cursor.frame < vmaid->select.frame ? 1 : 0) : -1;
   if (frame >= 0 && frame < vmaid->top) /* ja:選択開始位置が前のとき */
     signal_expose_text (widget, gc, vmaid, frame, 0, 0);
-  for (i = vmaid->top, rc0.x = vmaid->width + 2; i <= max;
-                                                i++, rc0. x += vmaid->width)
+  for (i = vmaid->top, rc0.x = vmaid->selfrm ? vmaid->width + 2 : 1; i <= max;
+                                                    i++, rc0.x += vmaid->width)
     signal_expose_text (widget, gc, vmaid, i, rc0. x, 0);
   if (vmaid->top + sx <= frame)         /* ja:選択開始位置が後のとき */
     signal_expose_text (widget, gc, vmaid, frame,
@@ -413,45 +413,53 @@ signal_expose (GtkWidget      *widget,
     {
       AviFrame *avi_frame;
 
-      avi_frame = avi_get_frame_open (vmaid->avi_edit[0]);
+      avi_frame = avi_frame_open (vmaid->avi_edit[0]);
       max = MIN (get_max_frame (vmaid, 0), vmaid->top + sx - 1);
       rc0.x = 0;
-      if (frame >= 0 && frame < vmaid->top)
+      if (vmaid->selfrm)
         {
-          /* ja:選択開始位置が前のとき */
-          rc0.width = vmaid->width;
-          if (gdk_rectangle_intersect (&event->area, &rc0, &rc1))
+          if (frame >= 0 && frame < vmaid->top)
             {
-              signal_expose_bitmap (widget, gc, avi_frame, frame,
+              /* ja:選択開始位置が前のとき */
+              rc0.width = vmaid->width;
+              if (gdk_rectangle_intersect (&event->area, &rc0, &rc1))
+                {
+                  signal_expose_bitmap (widget, gc, avi_frame, frame,
                                         rc0.x, rc0.y, rc0.width, rc0.height);
+                }
+              rc0.x = vmaid->width;
+              rc0.width = 2;
+            }
+          else
+            {
+              rc0.width = vmaid->width + 2;
             }
-          rc0.x = vmaid->width;
-          rc0.width = 2;
         }
       else
         {
-          rc0.width = vmaid->width + 2;
+          rc0.width = 1;
         }
       if (gdk_rectangle_intersect (&event->area, &rc0, &rc1))
         {
-          gdk_gc_set_foreground (gc,system_color + 1);
+          gdk_gc_set_foreground (gc, system_color + 1);
           gdk_draw_rectangle (widget->window, gc, TRUE, rc1.x, rc1.y,
                                                         rc1.width, rc1.height);
         }
       rc0.width = vmaid->width;
-      for (i = vmaid->top,rc0.x = vmaid->width + 2; i <= max;
-                                                    i++, rc0.x += vmaid->width)
+      for (i = vmaid->top, rc0.x = vmaid->selfrm ? vmaid->width + 2 : 1;
+                                        i <= max; i++, rc0.x += vmaid->width)
         if (gdk_rectangle_intersect (&event->area, &rc0, &rc1))
           signal_expose_bitmap (widget, gc, avi_frame, i,
                                         rc0.x, rc0.y, rc0.width, rc0.height);
-      rc0.width = vmaid->width * (sx + 1) + 4 - rc0.x;
+      rc0.width = (vmaid->selfrm ? vmaid->width + 4 : 2) + vmaid->width * sx
+                                                                    - rc0.x;
       if (gdk_rectangle_intersect (&event->area, &rc0, &rc1))
         {
-          gdk_gc_set_foreground (gc,system_color + 1);
+          gdk_gc_set_foreground (gc, system_color + 1);
           gdk_draw_rectangle (widget->window, gc, TRUE, rc1.x, rc1.y,
                                                         rc1.width, rc1.height);
         }
-      rc0.x = vmaid->width * (sx + 1) + 4;
+      rc0.x = (vmaid->selfrm ? vmaid->width + 4 : 2) + vmaid->width * sx;
       rc0.width = vmaid->width;
       if (vmaid->top + sx <= frame
                         && gdk_rectangle_intersect (&event->area, &rc0, &rc1))
@@ -469,7 +477,7 @@ signal_expose (GtkWidget      *widget,
                                         rc1.x, rc1.y, rc1.width, rc1.height);
         }
       rc0.y += vmaid->height;
-      avi_get_frame_close (avi_frame);
+      avi_frame_close (avi_frame);
     }
   if (vmaid->avi_edit[1])
     {
@@ -484,7 +492,7 @@ signal_expose (GtkWidget      *widget,
           gdk_draw_rectangle (widget->window, gc, TRUE,
                                         rc1.x, rc1.y, rc1.width, rc1.height);
         }
-      avi_pcm = avi_get_pcm_open (vmaid->avi_edit[1]);
+      avi_pcm = avi_pcm_open (vmaid->avi_edit[1]);
       max = MIN (get_max_frame (vmaid, 1), vmaid->top + sx - 1);
       rc0.x = 0;
       rc0.width = vmaid->width;
@@ -495,8 +503,8 @@ signal_expose (GtkWidget      *widget,
             (gint)((glonglong)frame * vmaid->scale * 1000 / vmaid->rate),
             (gint)((glonglong)(frame + 1) * vmaid->scale * 1000 / vmaid->rate),
                                         rc0.x, rc0.y, rc0.width, rc0.height);
-      for (i = vmaid->top, rc0.x = vmaid->width + 2; i <= max;
-                                                i++, rc0. x += vmaid->width)
+      for (i = vmaid->top, rc0.x = vmaid->selfrm ? vmaid->width + 2 : 1;
+                                        i <= max; i++, rc0.x += vmaid->width)
         if (gdk_rectangle_intersect (&event->area, &rc0, &rc1))
           signal_expose_wave (widget, gc, vmaid->avi_edit[1], avi_pcm,
                 (gint)((glonglong)i * vmaid->scale * 1000 / vmaid->rate),
@@ -510,7 +518,7 @@ signal_expose (GtkWidget      *widget,
             (gint)((glonglong)frame * vmaid->scale * 1000 / vmaid->rate),
             (gint)((glonglong)(frame + 1) * vmaid->scale * 1000 / vmaid->rate),
                                         rc0.x, rc0.y, rc0.width, rc0.height);
-      avi_get_pcm_close (avi_pcm);
+      avi_pcm_close (avi_pcm);
     }
   /* ja:選択範囲 */
   if (vmaid->select.stream >= 0)
@@ -523,7 +531,8 @@ signal_expose (GtkWidget      *widget,
       if (rc0.x <= rc0.width)
         {
           rc0.width = (rc0.width - rc0.x + 1) * vmaid->width;
-          rc0.x = (rc0.x - vmaid->top + 1) * vmaid->width + 2;
+          rc0.x = (rc0.x - vmaid->top) * vmaid->width
+                                    + (vmaid->selfrm ? vmaid->width + 2 : 1);
           rc0.y = MIN (vmaid->cursor.stream, vmaid->select.stream);
           rc0.height = MAX (vmaid->cursor. stream, vmaid->select.stream)
                                                                 - rc0.y + 1;
@@ -547,7 +556,8 @@ signal_expose (GtkWidget      *widget,
                                     && vmaid->top <= vmaid->cursor.frame
                                     && vmaid->cursor.frame <= vmaid->top + sx)
     {
-      rc0.x = (vmaid->cursor.frame - vmaid->top + 1) * vmaid->width + 1;
+      rc0.x = (vmaid->cursor.frame - vmaid->top) * vmaid->width
+                                    + (vmaid->selfrm ? vmaid->width + 1 : 0);
       rc0.y = system_font_height * 2
                             + (vmaid->avi_edit[0] && vmaid->cursor.stream > 0
                                                         ? vmaid->height : 0);
@@ -594,11 +604,11 @@ signal_button_press_draw (GtkWidget      *widget,
                 select = vmaid->select;
                 shift = event->state & GDK_SHIFT_MASK;
                 max = get_max_frame (vmaid, -1) + 1;
-                sx = MAX ((widget->allocation.width - 4) / vmaid->width - 2,
-                                                                            1);
+                sx = get_sx_frame (vmaid);
                 /* ja:クリックした位置にキャレットを移動する */
                 vmaid->cursor.frame = vmaid->top
-                                + (event->x - vmaid->width - 2) / vmaid->width;
+                        + (event->x - (vmaid->selfrm ? vmaid->width + 2 : 1))
+                                                                / vmaid->width;
                 if (vmaid->cursor.frame < vmaid->top)
                   vmaid->cursor.frame = vmaid->top;
                 else if (vmaid->top + sx < vmaid->cursor.frame)
@@ -673,21 +683,24 @@ signal_motion_notify_draw (VmaidWindow     *vmaid,
   cursor = vmaid->cursor;
   select = vmaid->select;
   top = vmaid->top;
-  sx = MAX ((vmaid->drawing->allocation.width - 4) / vmaid->width - 2, 1);
+  sx = get_sx_frame (vmaid);
   max = get_max_frame (vmaid, -1) + 1;
   ps.x = x;
   ps.y = y;
-  if (ps.x < vmaid->width + 2)
-    ps.x = vmaid->width + 2;
-  else if (ps.x > (sx + 1) * vmaid->width + 2)
+  if (ps.x < (vmaid->selfrm ? vmaid->width + 2 : 1))
+    ps.x = vmaid->selfrm ? vmaid->width + 2 : 1;
+  else if (ps.x > (vmaid->selfrm ? vmaid->width + 2 : 1)
+                                                    + sx * vmaid->width + 1)
     ps.x = (sx + 1) * vmaid->width + 2;
   /* ja:カーソルが外にあるときスクロールする */
-  if (x < vmaid->width + 2 && vmaid->top > 0)
+  if (x < (vmaid->selfrm ? vmaid->width + 2 : 1) && vmaid->top > 0)
     vmaid->top--;
-  else if (x > (sx + 1) * vmaid->width + 2 && vmaid->top < max - sx)
+  else if (x > (vmaid->selfrm ? vmaid->width + 2 : 1) + sx * vmaid->width + 1
+                                                    && vmaid->top < max - sx)
     vmaid->top++;
   /* ja:カーソルの位置にキャレットを移動する */
-  vmaid->cursor.frame = vmaid->top + (ps.x - vmaid->width - 2) / vmaid->width;
+  vmaid->cursor.frame = vmaid->top
+            + (ps.x - (vmaid->selfrm ? vmaid->width + 2 : 1)) / vmaid->width;
   if (vmaid->cursor.frame < 0)
     vmaid->cursor.frame = 0;
   else if (vmaid->cursor.frame > max)
@@ -751,7 +764,7 @@ signal_motion_notify (GtkWidget      *widget,
 
   if (event->is_hint)
     {
-      gdk_window_get_pointer(event->window,&x,&y,&state);
+      gdk_window_get_pointer (event->window, &x, &y, &state);
     }
   else
     {
@@ -759,7 +772,7 @@ signal_motion_notify (GtkWidget      *widget,
       y = event->y;
       state = event->state;
     }
-  if ((state&GDK_BUTTON1_MASK)
+  if ((state & GDK_BUTTON1_MASK)
                             && signal_motion_notify_draw (vmaid, x, y, state)
                                                     && vmaid->timer_id == 0)
     /* ja:はじめてスクロールしたとき */
@@ -791,13 +804,13 @@ signal_scroll (GtkWidget      *widget,
   gint max, sx, top;
 
   top = vmaid->top;
-  sx = MAX ((widget->allocation.width - 4) / vmaid->width - 2, 1);
+  sx = get_sx_frame (vmaid);
   max = get_max_frame (vmaid, -1) + 1;
   vmaid->top += event->direction == GDK_SCROLL_UP
                             || event->direction == GDK_SCROLL_LEFT ? -4 : 4;
   if (vmaid->top < 0)
     vmaid->top = 0;
-  else if (vmaid->top > max - sx)
+  else if (vmaid->top > MAX (max - sx, 0))
     vmaid->top = MAX (max - sx, 0);
   if (vmaid->top != top)
     misc_set_scroll_bar (vmaid->hscroll,
@@ -841,12 +854,12 @@ signal_key_press (GtkWidget   *widget,
           select = vmaid->select;
           top = vmaid->top;
           max = get_max_frame (vmaid, -1) + 1;
-          sx = MAX ((widget->allocation.width - 4) / vmaid->width - 2, 1);
+          sx = get_sx_frame (vmaid);
           vmaid->cursor.stream = vmaid->cursor.stream == 0 ? 1 : 0;
           /* ja:キャレットが表示されるように調整 */
-          if (vmaid->top > vmaid->cursor.frame)
+          if (vmaid->top > MIN (vmaid->cursor.frame, max - sx))
             vmaid->top = MIN (vmaid->cursor.frame, max - sx);
-          if (vmaid->top < vmaid->cursor.frame - sx)
+          if (vmaid->top < MAX (vmaid->cursor.frame - sx, 0))
             vmaid->top = MAX (vmaid->cursor.frame - sx, 0);
           if (vmaid->select.stream >= 0 && !(event->state & GDK_SHIFT_MASK))
             {
@@ -875,7 +888,6 @@ signal_key_press (GtkWidget   *widget,
       case GDK_L:
       case GDK_h:
       case GDK_l:
-        result=TRUE;
         {
           gboolean alt, ctrl;
           gint sx, max, top;
@@ -887,7 +899,7 @@ signal_key_press (GtkWidget   *widget,
           select = vmaid->select;
           top = vmaid->top;
           max = get_max_frame (vmaid, -1) + 1;
-          sx = MAX ((widget->allocation.width - 4) / vmaid->width - 2, 1);
+          sx = get_sx_frame (vmaid);
           alt = event->state & GDK_MOD1_MASK;
           ctrl = event->state & GDK_CONTROL_MASK;
           switch (event->keyval)
@@ -907,6 +919,7 @@ signal_key_press (GtkWidget   *widget,
                 if (!ctrl)
                   {
                     vmaid->cursor.frame--;
+                    break;
                   }
                 else if (vmaid->top > 0)
                   {
@@ -915,6 +928,17 @@ signal_key_press (GtkWidget   *widget,
                       vmaid->cursor.frame--;
                   }
                 break;
+              case GDK_comma:
+              case GDK_less:
+                if (!ctrl)
+                  goto loop;
+                if (vmaid->top > 0)
+                  {
+                    vmaid->top--;
+                    if (vmaid->cursor.frame-sx > vmaid->top)
+                      vmaid->cursor.frame--;
+                  }
+                break;
               case GDK_L:
               case GDK_l:
                 if (!alt)
@@ -931,6 +955,17 @@ signal_key_press (GtkWidget   *widget,
                       vmaid->cursor.frame++;
                   }
                 break;
+              case GDK_period:
+              case GDK_greater:
+                if (!ctrl)
+                  goto loop;
+                if (vmaid->top < max - sx)
+                 {
+                    vmaid->top++;
+                    if (vmaid->cursor.frame < vmaid->top)
+                      vmaid->cursor.frame++;
+                  }
+                break;
               case GDK_Page_Up:
                 if (ctrl)
                   {
@@ -945,7 +980,7 @@ signal_key_press (GtkWidget   *widget,
               case GDK_Page_Down:
                 if (ctrl)
                   {
-                    vmaid->cursor.frame = vmaid->top+sx;
+                    vmaid->cursor.frame = vmaid->top + sx;
                   }
                 else
                   {
@@ -954,13 +989,13 @@ signal_key_press (GtkWidget   *widget,
                   }
             }
           if (vmaid->cursor.frame < 0)
-            vmaid->cursor.frame=0;
+            vmaid->cursor.frame = 0;
           else if (vmaid->cursor.frame > max)
             vmaid->cursor.frame = max;
           /* ja:キャレットが表示されるように調整 */
-          if (vmaid->top > vmaid->cursor.frame)
+          if (vmaid->top > MIN (vmaid->cursor.frame, max - sx))
             vmaid->top = MIN (vmaid->cursor.frame, max - sx);
-          if (vmaid->top < vmaid->cursor.frame - sx)
+          if (vmaid->top < MAX (vmaid->cursor.frame - sx, 0))
             vmaid->top = MAX (vmaid->cursor.frame - sx, 0);
           if (vmaid->cursor.frame == cursor.frame && vmaid->top == top)
             break;/* ja:変化なし */
@@ -986,12 +1021,13 @@ signal_key_press (GtkWidget   *widget,
           clear_sel (vmaid, &cursor, &select, top);
           draw_caret (vmaid, &cursor);
         }
+        result = TRUE;
         break;
       case GDK_Escape:
       case GDK_Cancel:
-        result=TRUE;
+        result = TRUE;
         {
-          gint sx, top;
+          gint sx, max, top;
           VmaidCursor cursor, select;
 
           if (vmaid->select.stream < 0)
@@ -999,12 +1035,12 @@ signal_key_press (GtkWidget   *widget,
           cursor = vmaid->cursor;
           select = vmaid->select;
           top = vmaid->top;
-          sx = MAX ((widget->allocation.width - 4) / vmaid->width - 2, 1);
+          max = get_max_frame (vmaid, -1) + 1;
+          sx = get_sx_frame (vmaid);
           /* ja:キャレットが表示されるように調整 */
-          if (vmaid->top > vmaid->cursor.frame)
-            vmaid->top = MIN (vmaid->cursor.frame,
-                                        get_max_frame (vmaid, -1) + 1 - sx);
-          if (vmaid->top < vmaid->cursor.frame - sx)
+          if (vmaid->top > MIN (vmaid->cursor.frame, max - sx))
+            vmaid->top = MIN (vmaid->cursor.frame, max - sx);
+          if (vmaid->top < MAX (vmaid->cursor.frame - sx, 0))
             vmaid->top = MAX (vmaid->cursor.frame - sx, 0);
           vmaid->select.stream = -1;/* ja:選択解除 */
           /* ja:メニュー */
@@ -1051,16 +1087,16 @@ signal_key_press (GtkWidget   *widget,
         break;
       case GDK_F10:
         {
-          gint sx, top;
+          gint sx, max, top;
 
           if (event->state & GDK_SHIFT_MASK)
             break;
           top = vmaid->top;
-          sx = MAX ((widget->allocation.width - 4) / vmaid->width - 2, 1);
+          max = get_max_frame (vmaid, -1) + 1;
+          sx = get_sx_frame (vmaid);
           /* ja:キャレットが表示されるように調整 */
-          if (vmaid->top > vmaid->cursor.frame)
-            vmaid->top = MIN (vmaid->cursor.frame,
-                                        get_max_frame (vmaid, -1) + 1 - sx);
+          if (vmaid->top > MIN (vmaid->cursor.frame, max - sx))
+            vmaid->top = MIN (vmaid->cursor.frame, max - sx);
           if (vmaid->top < vmaid->cursor.frame - sx)
             vmaid->top = MAX (vmaid->cursor.frame - sx, 0);
           if (vmaid->top != top)
@@ -1079,11 +1115,10 @@ signal_key_press (GtkWidget   *widget,
           gtk_menu_popup (GTK_MENU (misc_find_menu (popup_entries, "/")),
                                     NULL, NULL, NULL, NULL, 3, event->time);
         }
-        result=TRUE;
+        result = TRUE;
         break;
       case GDK_Tab:
       case GDK_KP_Tab:
-        result=TRUE;
         {
           gint page;
           guint length;
@@ -1104,6 +1139,7 @@ signal_key_press (GtkWidget   *widget,
           gtk_widget_grab_focus (((VmaidWindow *)
                             orz_mdi_get_data (ORZ_MDI (mdi), page))->drawing);
         }
+        result = TRUE;
     }
   loop:
   return result;
