@@ -26,6 +26,7 @@
 #include "sigfile.h"
 #include "sigmain.h"
 #include "thread.h"
+#include "avicore/aviclip.h"
 #include "misc/argument.h"
 #include "misc/fileio.h"
 #include "misc/gnugpl.h"
@@ -41,9 +42,7 @@
 
 
 gboolean caret = FALSE;             /* ja:キャレットの形 */
-#if GTK_CHECK_VERSION(2,4,0)
 gboolean addext = TRUE;             /* ja:TRUE:拡張子を追加する */
-#endif /* GTK_CHECK_VERSION(2,4,0) */
 gboolean newfile = FALSE;           /* ja:TRUE:新規ファイル,FALSE:なし */
 gboolean second = FALSE;            /* ja:TRUE:新規,FALSE:既存 */
 gboolean default_ratio = FALSE;     /* ja:TRUE:縦/横の比率を保持する */
@@ -51,14 +50,12 @@ gboolean default_selfrm = TRUE;     /* ja:TRUE:選択されたフレーム表示
 gboolean file_merge = FALSE;        /* ja:TRUE:連番,FALSE:単独 */
 gchar *open_path = NULL;            /* ja:開くのパス */
 gchar *save_path = NULL;            /* ja:名前を付けて保存のパス */
-gchar *clipboard_scenario = NULL;   /* ja:クリップボードのテキスト */
 gint def_width, def_height;         /* ja:デフォルトのウインドウのサイズ */
 gint default_view = 6;              /* ja:ウインドウが作成時の初期値 */
 guint default_rate = 15;            /* ja:ウインドウが作成時の初期値 */
 guint timer_id = 0;                 /* ja:タイマID */
+GObject *clipboard;                 /* ja:クリップボード */
 GObject *history;                   /* ja:ファイルの履歴 */
-GdkAtom atom_targets;               /* ja:ターゲット */
-GdkAtom atom_scenario;              /* ja:シナリオオブジェクト */
 GdkBitmap *bitmap0;                 /* ja:波背景 */
 GdkBitmap *bitmap1;                 /* ja:網背景 */
 GtkWidget *window;                  /* ja:ウインドウ */
@@ -71,11 +68,8 @@ gint system_font_height;            /* ja:フォントの高さ */
 gint system_font_ascent;            /* ja:フォントのアクセントの高さ */
 GdkColor system_color[4] = {{0, 0x0000, 0x0000, 0x0000},
                             {0, 0xffff, 0xffff, 0xffff},
-                            {0, 0xffff, 0xffff, 0xffff},
+                            {0, 0x8000, 0x8000, 0x8000},
                             {0, 0x0000, 0x0000, 0x8000}};
-#ifdef G_OS_WIN32
-UINT uFormat = 0;                   /* ja:クリップボード */
-#endif /* G_OS_WIN32 */
 MiscCreateMenuEntry menu_entries[] = {
 {"/",                   NULL,                  NULL,          NULL,             MISC_CREATE_MENU_BAR,                               NULL,             NULL, NULL},
 {"/file",               N_("_File"),           NULL,          NULL,             MISC_CREATE_MENU_ITEM,                              NULL,             NULL, NULL},
@@ -110,7 +104,7 @@ MiscCreateMenuEntry menu_entries[] = {
 {"/edit/jump",          GTK_STOCK_JUMP_TO,     "<control>J",  NULL,             MISC_CREATE_MENU_STOCK | MISC_CREATE_MENU_ELLIPSIS, command_jump,     NULL, NULL},
 {"/edit/sep4",          NULL,                  NULL,          NULL,             MISC_CREATE_MENU_SEPARATOR,                         NULL,             NULL, NULL},
 #if GTK_CHECK_VERSION(2,10,0)
-{"/edit/all",           GTK_STOCK_SELECT_ALL,  "<control>A",  NULL,             MISC_CREATE_MENU_ITEM,                              command_all,      NULL, NULL},
+{"/edit/all",           GTK_STOCK_SELECT_ALL,  "<control>A",  NULL,             MISC_CREATE_MENU_STOCK,                             command_all,      NULL, NULL},
 #else /* not GTK_CHECK_VERSION(2,10,0) */
 {"/edit/all",           N_("_All"),            "<control>A",  select_all16_xpm, MISC_CREATE_MENU_ITEM,                              command_all,      NULL, NULL},
 #endif /* not GTK_CHECK_VERSION(2,10,0) */
@@ -144,7 +138,7 @@ MiscCreateMenuEntry popup_entries[] = {
 {"/delete",             GTK_STOCK_DELETE,      NULL,          NULL,             MISC_CREATE_MENU_STOCK,                             command_delete,   NULL, NULL},
 {"/sep2",               NULL,                  NULL,          NULL,             MISC_CREATE_MENU_SEPARATOR,                         NULL,             NULL, NULL},
 #if GTK_CHECK_VERSION(2,10,0)
-{"/all",                GTK_STOCK_SELECT_ALL,  "<control>A",  NULL,             MISC_CREATE_MENU_ITEM,                              command_all,      NULL, NULL},
+{"/all",                GTK_STOCK_SELECT_ALL,  "<control>A",  NULL,             MISC_CREATE_MENU_STOCK,                             command_all,      NULL, NULL},
 #else /* not GTK_CHECK_VERSION(2,10,0) */
 {"/all",                N_("_All"),            "<control>A",  select_all16_xpm, MISC_CREATE_MENU_ITEM,                              command_all,      NULL, NULL},
 #endif /* not GTK_CHECK_VERSION(2,10,0) */
@@ -181,7 +175,7 @@ int
 main (int   argc,
       char *argv[])
 {
-  gboolean *arg_files;
+  gboolean *arg_files, success[4];
   gchar *key, *utf8str, *data, *user_config = NULL;
 #if ! GLIB_CHECK_VERSION(2,6,0) && defined (G_OS_WIN32)
   LPITEMIDLIST pidl = NULL;
@@ -207,9 +201,6 @@ main (int   argc,
   GtkWidget *text, *handle, *scroll, *vbox;
   /* en:Drag&Drop */
   GtkTargetEntry drag_types[1] = {{"text/uri-list", 0, 0}};
-  /* ja:Video maidが提供可能なセレクション */
-  GtkTargetEntry targets[1]
-                        = {{"Video maid Scenario Object", 0, TARGET_VMAID}};
   static gchar *xpm0[] = {
 "4 8 2 1",
 "   c None",
@@ -248,7 +239,7 @@ main (int   argc,
   locale = g_build_filename (dirname, "locale", NULL);
   g_free (dirname);
 # ifdef G_OS_WIN32
-  if (!fileio_isfile (locale))
+  if (!g_file_test (locale, G_FILE_TEST_EXISTS))
     {
       g_free (locale);
       locale = g_win32_get_package_installation_subdirectory
@@ -320,9 +311,7 @@ main (int   argc,
       license = g_key_file_get_integer (key_file, "vmaid", "license", NULL);
       orz_history_set_num (ORZ_HISTORY (history),
                 g_key_file_get_integer (key_file, "vmaid", "history", NULL));
-#if GTK_CHECK_VERSION(2,4,0)
       addext = g_key_file_get_boolean (key_file, "vmaid", "addext", NULL);
-#endif /* GTK_CHECK_VERSION(2,4,0) */
       newfile = g_key_file_get_boolean (key_file, "vmaid", "newfile", NULL);
       second = g_key_file_get_boolean (key_file, "vmaid", "second", NULL);
       def_width = g_key_file_get_integer (key_file, "vmaid", "def_width",
@@ -443,21 +432,10 @@ VMAID_COPYRIGHT_NOTICE"\n"
                                             drag_types, 1, GDK_ACTION_COPY);
   g_signal_connect (G_OBJECT (window), "drag-data-received",
                                 G_CALLBACK (signal_drag_data_received), NULL);
-#ifdef G_OS_WIN32
   /* ja:クリップボード */
-  uFormat = RegisterClipboardFormat (_T("Video maid Scenario Object"));
-#endif /* G_OS_WIN32 */
-  /* ja:クリップボード */
-  atom_targets = gdk_atom_intern ("TARGETS", FALSE);
-  atom_scenario = gdk_atom_intern ("Video maid Scenario Object", FALSE);
-  gtk_selection_add_targets (window, GDK_SELECTION_CLIPBOARD,
-                                            targets, G_N_ELEMENTS (targets));
-  g_signal_connect (G_OBJECT (window), "selection-clear-event",
-                                G_CALLBACK (signal_selection_clear), NULL);
-  g_signal_connect (G_OBJECT(window), "selection-get",
-                                G_CALLBACK (signal_selection_get), NULL);
-  g_signal_connect (G_OBJECT (window), "selection-received",
-                                G_CALLBACK (signal_selection_received), NULL);
+  clipboard = avi_clip_new (window);
+  g_signal_connect (G_OBJECT (clipboard), "paste",
+                                    G_CALLBACK (signal_selection_paste), NULL);
   /* en:MDI */
   mdi = orz_mdi_new ();
   gtk_notebook_set_scrollable (GTK_NOTEBOOK (mdi), TRUE);
@@ -639,6 +617,8 @@ VMAID_COPYRIGHT_NOTICE"\n"
   system_color[1] = style->base[0];
   system_color[2] = style->dark[0];
   system_color[3] = style->bg[3];
+  gdk_colormap_alloc_colors (gdk_colormap_get_system (),
+                                        system_color, 4, FALSE, TRUE, success);
   /* ja:システムフォント */
   system_font = gdk_font_ref (gtk_style_get_font (style));
   for (i = 0; i < 256; i++)
@@ -695,9 +675,7 @@ VMAID_COPYRIGHT_NOTICE"\n"
   g_key_file_set_integer (key_file, "vmaid", "license", license);
   g_key_file_set_integer (key_file, "vmaid", "history",
                                 orz_history_get_num (ORZ_HISTORY (history)));
-#if GTK_CHECK_VERSION(2,4,0)
   g_key_file_set_boolean (key_file, "vmaid", "addext", addext);
-#endif /* GTK_CHECK_VERSION(2,4,0) */
   g_key_file_set_boolean (key_file, "vmaid", "newfile", newfile);
   g_key_file_set_boolean (key_file, "vmaid", "second", second);
   g_key_file_set_integer (key_file, "vmaid", "def_width", def_width);
@@ -757,7 +735,6 @@ VMAID_COPYRIGHT_NOTICE"\n"
 
   gdk_font_unref (system_font);
   g_free (user_config);
-  g_free (clipboard_scenario);
   g_free (open_path);
   g_free (save_path);
   gtk_exit (0);
