@@ -1,6 +1,6 @@
 /*
     Video maid
-    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2006 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -35,7 +35,7 @@ extern "C" {
 
 
 #define VMAID_VERSION_NOTICE "Video maid "VERSION" ("BUILD_ENVIRONMENT")"
-#define VMAID_COPYRIGHT_NOTICE "Copyright (C) 1998-2005 Kazuki IWAMOTO"
+#define VMAID_COPYRIGHT_NOTICE "Copyright (C) 1998-2006 Kazuki IWAMOTO"
 
 #define MENUFILE 9
 
@@ -53,6 +53,7 @@ typedef struct _VmaidHistory
 } VmaidHistory;
 typedef struct _VmaidWindow
 {
+  gboolean selfrm;              /* ja:選択されたフレーム表示 */
   gint top;                     /* ja:ウインドウの先頭 */
   gint width, height;           /* ja:フレームのサイズ */
   guint rate, scale;            /* ja:レート÷スケール=1秒間のフレーム数 */
@@ -66,27 +67,21 @@ typedef struct _VmaidWindow
 } VmaidWindow;
 
 
-enum
-{
-  TARGET_VMAID
-};
-
-
 extern gboolean caret;                  /* ja:キャレットの形 */
+extern gboolean addext;                 /* ja:TRUE:拡張子を追加する */
 extern gboolean newfile;                /* ja:TRUE:新規ファイル,FALSE:なし */
 extern gboolean second;                 /* ja:TRUE:新規,FALSE:既存 */
 extern gboolean default_ratio;          /* ja:TRUE:縦/横の比率を保持する */
+extern gboolean default_selfrm;         /* ja:TRUE:選択されたフレーム表示 */
 extern gboolean file_merge;             /* ja:TRUE:連番,FALSE:単独 */
 extern gchar *open_path;                /* ja:開くのパス */
 extern gchar *save_path;                /* ja:名前を付けて保存のパス */
-extern gchar *clipboard_scenario;       /* ja:クリップボードのテキスト */
 extern gint def_width,def_height;       /* ja:デフォルトのウインドウのサイズ */
 extern gint default_view;               /* ja:ウインドウが作成時の初期値 */
 extern guint default_rate;              /* ja:ウインドウが作成時の初期値 */
 extern guint timer_id;                  /* ja:タイマID */
+extern GObject *clipboard;              /* ja:クリップボード */
 extern GObject *history;                /* ja:ファイルの履歴 */
-extern GdkAtom atom_targets;            /* ja:ターゲット */
-extern GdkAtom atom_scenario;           /* ja:シナリオオブジェクト */
 extern GdkBitmap *bitmap0;              /* ja:波背景 */
 extern GdkBitmap *bitmap1;              /* ja:網背景 */
 extern GtkWidget *window;               /* ja:ウインドウ */
@@ -97,14 +92,11 @@ extern GtkPositionType n_pos;           /* ja:ノートブックの位置 */
 extern GdkFont *system_font;            /* ja:フォント */
 extern gint system_font_height;         /* ja:フォントの高さ */
 extern gint system_font_ascent;         /* ja:フォントのアクセントの高さ */
-extern GtkListStore *list_store;        /* ja:リスト */
 extern GdkColor system_color[4];
-#ifdef G_OS_WIN32
-extern UINT uFormat;                    /* ja:クリップボード */
-#endif /* G_OS_WIN32 */
 extern MiscCreateMenuEntry menu_entries[];
 extern MiscCreateMenuEntry popup_entries[];
 extern MiscCreateMenuEntry tview_entries[];
+extern MiscCreateToolbarEntry toolbar_entries[];
 
 
 #ifdef __cplusplus
